/*
 * =====================================================================================
 *
 *       Filename:  vcomm.c
 *
 *    Description:  Virtual Comm based on STR91x stack
 *
 *       Version:  1.0
 *       Created:  20 mar 2008 13:57:12
 *       Revision:  none
 *       Compiler:  gcc
 *       Licence: GPL v3
 *       Author:  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>
 *       Company:  BoFF
 *
 * =====================================================================================
 */
#include "usb_lib.h"
#include "usb_desc.h"
#include "hw_config.h"
#include "vcomm.h"
#include "armint.h"
#include "usb_lib.h"
#include "usb_conf.h"
#include "usb_prop.h"
#include "usb_pwr.h"
#include "hw_config.h"

#define MAX_PACKET_SIZE 64

#define MIN(X, Y)  ((X) < (Y) ? (X) : (Y))

//Bulk buffer
static u8 bulkBuf[MAX_PACKET_SIZE];

//Fifo rx and tx buffers
static u8 txdata[VCOM_FIFO_RX_SIZE];
static u8 rxdata[VCOM_FIFO_TX_SIZE];

//Global fifo data
volatile static u16 txcnt,rxpos,txpos,rxcnt;

/*-----------------------------------------------------------*/
//RX data
void EP3_OUT_Callback(void)
{
    int i,iLen;
    if ((VCOM_FIFO_RX_SIZE-rxcnt) < MAX_PACKET_SIZE)
    {
       // may not fit into fifo
      SetEPRxValid(ENDP3);
      return;
    }
    iLen = GetEPRxCount(ENDP3);
    PMAToUserBufferCopy(bulkBuf, ENDP3_RXADDR, iLen);
    for (i = 0; i < iLen; i++)
    {
       // put into FIFO
       rxdata [(rxpos + rxcnt++) % VCOM_FIFO_RX_SIZE] = bulkBuf[i];
    }
    SetEPRxValid(ENDP3);
}

/*-----------------------------------------------------------*/
//TX data
void EP1_IN_Callback(void)
{
    int i, iLen;
    if (!txcnt)
    {
        return;
    }
    // get bytes from transmit FIFO into intermediate buffer
    for (i = 0; i < MIN(MAX_PACKET_SIZE,txcnt) ; i++)
    {
        txcnt--;
        bulkBuf[i] = txdata[txpos++];
        if(txpos >= VCOM_FIFO_TX_SIZE) txpos = 0;
    }
    iLen = i;
    // send over USB
    if (iLen > 0)
    {
        UserToPMABufferCopy(bulkBuf, ENDP1_TXADDR, iLen);
        SetEPTxCount(ENDP1,iLen);
        SetEPTxValid(ENDP1);
    }
}

/*-----------------------------------------------------------*/

int vcom_putchar(int c)
{
    if(txcnt > VCOM_FIFO_TX_SIZE) return EOF;
    u8 uirq = (txcnt==0);
    cpu_t is = disable_irq();
    txdata[(txpos + txcnt++) % VCOM_FIFO_TX_SIZE] = c;
    restore_irq(is);
    if(uirq)
    {
        SetEPTxCount(ENDP1,0);
        SetEPTxValid(ENDP1);
    }
    return c;
}

/*-----------------------------------------------------------*/
int vcom_getchar(void)
{
    u8 c;
    if(!rxcnt) return EOF;
    cpu_t is = disable_irq();
    rxcnt--;
    c = rxdata[rxpos++];
    if (rxpos >= VCOM_FIFO_RX_SIZE) rxpos = 0;
    restore_irq(is);
    return c;
}

/*-----------------------------------------------------------*/
int vcom_rxbytes(void)
{
    return rxcnt;
}

/*-----------------------------------------------------------*/

void vcom_flush(void)
{
    cpu_t is = disable_irq();
    rxcnt = 0;
    rxpos = 0;
    txcnt = 0;
    txpos = 0;
    restore_irq(is);
}

/*-----------------------------------------------------------*/
void vcom_init(void)
{
  Set_System();
  USB_Interrupts_Config();
  USB_Init();
}
/*-----------------------------------------------------------*/

