/*
    Str912 book advanced example 2 - Bluetooth Module
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "uart.h"
#include "armint.h"
#include <string.h>
#include <stdlib.h>
#include <stdint.h>

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7 i 3
    SCU->PRR1 |= __GPIO7 | __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 7 i 3
    SCU->PCGR1 |= __GPIO7 | __GPIO3;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port 7 jako wyjsciowy
    GPIO7->DDR = 0xff;
    
    //Caly port P3.4..P3.7 jako wejscie
    SCU->GPIOIN[3] &= ~0xf0;
    
    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;
}


/*----------------------------------------------------------------------*/
//Rozmiar bufora inbuf
#define INBUF_SIZE 255

//Bufor zawierajacy znaki wejsciowe
char inBuf[INBUF_SIZE+1];

//Licznik znakow bufora
uint8_t inBufCnt;


/*----------------------------------------------------------------------*/
//Debug function
#define DBG(X) uartWriteString(UART0,X)

//Write to BT
#define BTW(X) uartWriteString(UART1,X)

//Abort
#define abort() do {  uartWriteString(UART0,"Aborted "); \
					  uartWriteString(UART0,__PRETTY_FUNCTION__);\
					  uartWriteString(UART0,"\r\n");\
					  while(1); \
				   } while(0)


/*----------------------------------------------------------------------*/
//Current editor mode
enum tMode 
{ 
  tModeCmd,			//Bluetooth connection Mode
  tModeConn 		//Bluetooth transfer Mode
};

/*----------------------------------------------------------------------*/
//Function return mode
enum stReturn 
{ 
  stNone,			//Not change mode
  stChange			//Chage mode
};

/*----------------------------------------------------------------------*/
//Get line Connect
enum stReturn newLineConnEvent(char *line)
{
	//Jezeli koniec linii to pobierz bufor
	DBG(line);
	if(strstr(line,"DISCONNECT")!=NULL)
	{
		return stChange;
	}
	char *arg;
	if ((arg=strstr(inBuf,"setled"))!=NULL)
	{
		int leds = atoi(arg+7);
		if(leds)
		{
			GPIO7->DR[0xff<<2] = leds;
			BTW("Diody zostaly ustawione pomyslnie !!\n> ");
		}
		else
		{
			BTW("Bledny argument komendy !!\n> ");
		}
	}
	else if((arg=strstr(inBuf,"key"))!=NULL)
	{
		BTW("> Klawisz K1 - ");
		if(!GPIO3->DR[0x10<<2])
		{
			BTW("WCISNIETY\n> ");
		}
		else
		{
			BTW("PUSZCZONY\n> ");
		}
		BTW("Klawisz K2 - ");
		if(!GPIO3->DR[0x20<<2])
		{
			BTW("WCISNIETY\n> ");
		}
		else
		{
			BTW("PUSZCZONY\n> ");
		}
		BTW("Klawisz K3 - ");
		if(!GPIO3->DR[0x40<<2])
		{
			BTW("WCISNIETY\n> ");
		}
		else
		{
			BTW("PUSZCZONY\n> ");
		}
		BTW("Klawisz K4 - ");
		if(!GPIO3->DR[0x80<<2])
		{
			BTW("WCISNIETY\n> ");
		}
		else
		{
			BTW("PUSZCZONY\n> ");
		}
	}
	else
	{
		BTW("Nieznana komenda !!\n> ");
	}

	return stNone;
}


/*----------------------------------------------------------------------*/
//Get line Init
enum stReturn newLineCmdEvent(char *line)
{
	//Jezeli koniec linii to pobierz bufor
	DBG(line);
	if(strstr(line,"CONNECT")!=NULL)
	{
		BTW("*** Mikrokontrolery STR912  - UART bez systemu przerwan***\n");
		BTW("*** (C) Lucjan Bryndza 2007. Licencja: GPLv3 ***\n");
		BTW("Program steruje diodami LED i sprawdza stan klawiszy\n");
		BTW("Aby zaswiecic lub wylaczyc diody wpisz setled [0-255]\n");
		BTW("Aby sprawdzic stan wcisnietych klawiszy wpisz key\n");
		BTW("\n>");	
		return stChange;
	}
	else
	{
		return stNone;
	}
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Inicjalizacja portu szeregowego
    uartInit(UART0,115200,15,128);
	
    //Inicjalizacja UART1
    uartInit(UART1,19200,14,256);
    
    //Zalacz przerwania
    enable_irq();
    
    //Stan aplikacji
    static enum tMode appState;
    
    
/*
    while(1)
    {
    	int u0 = uartReadByte(UART0);
    	if(u0!=UART_EMPTY) uartWriteByte(UART1,u0);
    	u0 = uartReadByte(UART1);
    	if(u0!=UART_EMPTY) uartWriteByte(UART0,u0);
    }
*/
    
    //Main loop
	while(1)
	{
		//Odbierz znak
		int zn = uartReadByte(UART1);
		if(zn!=UART_EMPTY)
		{
			if(appState==tModeConn ) uartWriteByte(UART1,zn);
			if(zn=='\r')
			{
				//Jezeli koniec linii to pobierz bufor
				inBuf[inBufCnt] = 0;
				inBufCnt = 0;
				switch(appState)
				{
				//Command Mode
				case tModeCmd:
					if(newLineCmdEvent(inBuf)==stChange) appState = tModeConn;
					break;
				//Connection Mode
				case tModeConn:
					if(newLineConnEvent(inBuf)==stChange) appState = tModeCmd;
					break;
				}
			}
			else
			{
				//Wez kolejny znak
				if(inBufCnt<INBUF_SIZE) inBuf[inBufCnt++] = zn;
			}
		}
	}	
    return 0;
}

