/*
    Str912 book example10 - Uart with interrupt
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdlib.h>
#include <string.h>
#include "uart.h"
#include "armint.h"


/*-------------------------------------------------------------*/
//Struktura prywatna zawierajaca bufory
typedef struct uartBuf
{
    uint8_t *rxBuf;
    int maxSize;
    volatile int rxCnt;
    volatile int rxPos;
    uint8_t *txBuf;
    volatile int txCnt;
    volatile int txPos;
} uartBuf;


//Struktura przechowujaca struktury prywatne
static uartBuf uartData[2];

/*-------------------------------------------------------------*/

//Glowna procedura obslugi przerwania
static void uartIRQ(UART_TypeDef *uartBase,uartBuf *buf)
{
    uint16_t id_int = uartBase->MIS;
    //Fifo Rx interrupt
    if( (id_int & (1<<4)) || id_int & (1<<6) )
    {
	//odczytuj dane z fifo dopuki sa
	while(!(uartBase->FR & (1<<4)))
	{
		if(buf->rxCnt < buf->maxSize)
		{
			buf->rxBuf[(buf->rxPos + buf->rxCnt++) % buf->maxSize] = uartBase->DR & 0xff;
		}
		else
		{
		    buf->rxBuf[buf->maxSize-1] = uartBase->DR & 0xff;
		}
	}
	//Kasuj zrodlo przerwania
	uartBase->ICR = (1<<4) | (1<<6);
    }
    //Fifo Tx interrupt
    if( id_int & (1<<5) )
    {
	//Mozna nadawac nowy znak
	for(;buf->txCnt && !(uartBase->FR & (1<<5)); buf->txCnt--)
	{
      	    uartBase->DR = buf->txBuf[buf->txPos++];
	    if(buf->txPos >= buf->maxSize) buf->txPos = 0;
	}
	//Kasuj zrodlo przerwania
	if(buf->txCnt==0) uartBase->ICR = 1<<5;
    }
}

/*-------------------------------------------------------------*/
//Przerwanie niemaskowalne fiq
static void uart0IRQHandler(void) __attribute__ ((interrupt("IRQ")));

void uart0IRQHandler(void)
{
    //Funkcja obslugi przerwania
    uartIRQ(UART0,&uartData[0]);
    //Informacja dla VIC (koniec przerwania)
    VIC1->VAR = 0;
}

/*-------------------------------------------------------------*/
//Przerwanie niemaskowalne fiq
static void uart1IRQHandler(void) __attribute__ ((interrupt("IRQ")));

void uart1IRQHandler(void)
{
    //Funkcja obslugi przerwania
    uartIRQ(UART1,&uartData[1]);
    //Informacja dla VIC (koniec przerwania)
    VIC1->VAR = 0;
}

/*-------------------------------------------------------------*/
#define VIC1_UART0 (1<<0)
#define VIC1_UART0_BIT 0
#define VIC1_UART1 (1<<1)
#define VIC1_UART1_BIT 1
#define VIC_SLOT_EN (1<<5)

//Initialize UAR
int uartInit(UART_TypeDef *uartBase,int baud,int irqPrio,int bufSize)
{
		
	//Jezeli prioritet za duzy to wyjdz
	if(irqPrio>15) return -1;
	//Zalaczenie clocka dla VIC
	SCU->PCGR0 |= __VIC;
	//Zwolnienie ewentualnego resetu
	SCU->PRR0 |= __VIC;
	//Ustawienia uartow
	if(uartBase==UART0)
	{
		//Odblokuj zegar i reset dla UART0
		SCU->PRR1 |= __GPIO3 | __UART0;
    	SCU->PCGR1 |= __GPIO3 | __UART0;
		uartBase = UART0;
		//Ustaw porty tak aby RX bylo wejsciem a TX wyjsciem
		SCU->GPIOOUT[3] |= (2<<(1*2));
		SCU->GPIOIN[3] |= (1<<0);
		GPIO3->DDR |= (1<<1);
		//Do wektora podstaw adres procedury obslugi
		VIC1->VAiR[irqPrio] = (unsigned long)uart0IRQHandler;
		//Ustaw slot na przerwanie od UART0
		VIC1->VCiR[irqPrio] = VIC1_UART0_BIT | VIC_SLOT_EN;
		//Zakwalifikuj przerwanie WIU jako IRQ
		VIC1->INTSR &= ~VIC1_UART0;
		//Vic interrupt enable register
		VIC1->INTER = VIC1_UART0;
		//Alokacja pamieci
		uartData[0].txBuf = (uint8_t*)malloc(bufSize);
		uartData[0].rxBuf = (uint8_t*)malloc(bufSize);
		uartData[0].maxSize = bufSize;
	}
	else if(uartBase==UART1)
	{
		//Odblokuj zegar i reset dla UART1
		SCU->PRR1 |= __GPIO3 | __UART1;
		SCU->PCGR1 |= __GPIO3 | __UART1;
		//Adres bazowt portu szeregowego
		uartBase = UART1;
		//Ustaw porty tak aby RX bylo wejsciem a TX wyjsciem
		SCU->GPIOOUT[3] |= (2<<(3*2));
		SCU->GPIOIN[3] |= (1<<2);
		GPIO3->DDR |= (3<<1);
		//Do wektora podstaw adres procedury obslugi
		VIC1->VAiR[irqPrio] = (unsigned long)uart1IRQHandler;
		//Ustaw slot na przerwanie od UART0
		VIC1->VCiR[irqPrio] = VIC1_UART1_BIT | VIC_SLOT_EN;
		//Zakwalifikuj przerwanie WIU jako IRQ
		VIC1->INTSR &= ~VIC1_UART1;
		//Vic interrupt enable register
		VIC1->INTER = VIC1_UART1;
		//Alokacja pamieci
		uartData[1].txBuf = (uint8_t*)malloc(bufSize);
		uartData[1].rxBuf = (uint8_t*)malloc(bufSize);
		uartData[1].maxSize = bufSize;
	}
	else
	{
		return -1;
	}
	//Wyznaczanie mnoznika i czesciowego podzielnika
	baud = 64 * (MAIN_CLOCK /2) / (16 * baud);
	uartBase->IBRD = baud >> 6; 
	uartBase->FBRD = baud & 0x3f;
	//Konfiguruj urzadzenie z wlaczonym fifo
	uartBase->LCR = UART_WordLength_8D |UART_FIFO_Enable;
	//Wlacz nadawanie i odbieranie znakow
	uartBase->CR = UART_Mode_Tx_Rx | 1;
	//Wlacz przerwania na porcie szeregowym RX TX Timeout
	uartBase->IMSC = 1<<4 | 1<<5 | 1<<6;
	
	return 0;
}


/*-------------------------------------------------------------*/
//Nadawanie znaku
void uartWriteByte(UART_TypeDef *uartBase,uint8_t byte)
{
	 uartBuf *buf = NULL;
	 if(uartBase==UART0) buf = &uartData[0];
	 else if(uartBase==UART1) buf = &uartData[1];
	 else return;
	 //Gdy wszystkie bajty, czekaj na zwolnienie
	 while(buf->txCnt >= buf->maxSize);

     //Wylacz przerwanie RS
     cpu_t irqs = disable_irq();
     uartBase->IMSC = 0;
	 restore_irq(irqs);

     if( !(uartBase->FR & 0x20)  && buf->txCnt==0 )
	 {
	    uartBase->DR = byte;
	 }
	 //Wpisz wszystko do bufora
	 else
	 {
	    //Wylacz przerwania
		buf->txBuf[(buf->txPos + buf->txCnt++) % buf->maxSize] = byte;
     }
    //Wlaczenie przerwan od RX TX
	irqs = disable_irq();
	uartBase->IMSC = 1<<4 | 1<<5 | 1<<6;
	restore_irq(irqs);
}

/*-------------------------------------------------------------*/
//Odebranie znaku
int uartReadByte(UART_TypeDef *uartBase)
{
	int c;
	uartBuf *buf = NULL;
	if(uartBase==UART0) buf = &uartData[0];
	else if(uartBase==UART1) buf = &uartData[1];
	else return -1;
	//Czekaj az bedzie znak
	if (buf->rxCnt==0) return UART_EMPTY;
	//Wylacz na moment przerwanie od RS
	cpu_t irqs = disable_irq();
	uartBase->IMSC = 0;
	restore_irq(irqs);
	
	buf->rxCnt--;
  	c = buf->rxBuf[buf->rxPos++];
  	if (buf->rxPos >= buf->maxSize) buf->rxPos = 0; 
	
	//Wlaczenie przerwan od RX TX
	irqs = disable_irq();
	uartBase->IMSC = 1<<4 | 1<<5 | 1<<6;
	restore_irq(irqs);
	return c;
}


/*-------------------------------------------------------------*/
//Wyslij tekst do portu szeregowego 
void uartWriteString(UART_TypeDef *uartBase,const char *string)
{
	while(*string)
	{
		if(*string == '\n') uartWriteByte(uartBase,'\r');
		uartWriteByte(uartBase,*string++);
	}
}

/*-------------------------------------------------------------*/
//Wylacz uart
int uartClose(UART_TypeDef *uartBase)
{
    if(uartBase==UART0) 
    {
	if(uartData[0].maxSize==0) return -1;
    }
    else if(uartBase==UART1) 
    {
	if(uartData[1].maxSize==0) return -1;
    }
    else
    {
	return -1;
    }
    //Wylacz przerwania
    cpu_t saveIrq = disable_irq();
    //Wylacz odpowiednie przerwanie
    if(uartBase==UART0) VIC1->INTECR = VIC1_UART0;
    else if(uartBase==UART1) VIC1->INTECR = VIC1_UART1;
    //Odblokuj stan przerwan
    restore_irq(saveIrq);
    //Zwolnij zajeta pamiec
    if(uartBase==UART0)
    {
	free(uartData[0].txBuf);
	free(uartData[0].rxBuf);
	memset(&uartData[0],0,sizeof(uartBuf));
    }
    else if(uartBase==UART1)
    {
	free(uartData[1].txBuf);
	free(uartData[1].rxBuf);
	memset(&uartData[1],0,sizeof(uartBuf));
    }
    return 0;
}
