/*
    Str912 book ADVANCED EXAMPLE  3 - Sys Clock for LWIP
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "clock-arch.h"
#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include <stdint.h>
#include "armint.h"
#include "uart.h"
/*---------------------------------------------------------------------------*/


//RTC timer interrupt interval enable
#define RTC_CR_PIE (1<<21)

//Timer interrupt 128HZ
#define RTC_CR_PISEL_128 (4<<16)

//Timer periodic interrupt flag
#define RTC_SR_PISF (1<<31)

//Bity kontrolera VIC
#define VIC1_RTC (1<<8)
#define VIC1_RTC_BIT 8

//Zalaczenie slotu
#define VIC_SLOTEN (1<<5)

/*---------------------------------------------------------------------------*/

//Global clock variable
static volatile clock_time_t clock;

/*---------------------------------------------------------------------------*/
//Przerwanie od ukladu czasowo licznikowego
static void irq_RTC_handler(void) __attribute__ ((interrupt("IRQ")));

void irq_RTC_handler(void)
{
    //Jezeli to przerwanie PERIODIC
    if(RTC->SR & RTC_SR_PISF)
    {
        clock++;
        //Zeruj flagi przerwan w RTC
        RTC->SR |= RTC_SR_PISF;
    }
    
    //Informacja do VIC - koniec przerwania
    VIC1->VAR = 0;
}

/*---------------------------------------------------------------------------*/
//Get time clock
clock_time_t clock_time(void)
{
	return clock;
}

/*---------------------------------------------------------------------------*/
//Initialize system clock
void clock_init(void)
{
	/*** Ustawienie Timera 0 w tryb czasomierza ****/
	//Odblokowanie CLK i RST
	SCU->PCGR1 |= __RTC;
	SCU->PRR1  |= __RTC;
	/**** Ustawienia dla RTC *******/
	RTC->CR = RTC_CR_PISEL_128;
	/**** Ustawienie kontrolera przerwan VIC ***/
	//Zalaczenie clocka dla VIC
	SCU->PCGR0 |= __VIC;
	//Reset
	SCU->PRR0 &= ~__VIC;
	SCU->PRR0 |= __VIC;
	//Do wektora podstaw adres procedury obslugi
	VIC1->VAiR[15] = (unsigned long)irq_RTC_handler;
	//Ustaw sOClot na przerwanie od WIUOC
	VIC1->VCiR[15] = VIC1_RTC_BIT | VIC_SLOTEN;
	//Zakwalifikuj przerwanie jako IRQ
	VIC1->INTSR &= ~VIC1_RTC;
	//Vic interrupt enable register
	VIC1->INTER |= VIC1_RTC;
	//Przerwanie od RTC alarm and periodic
	RTC->CR |= RTC_CR_PIE;
}

/*---------------------------------------------------------------------------*/
