/*
    Str912 book ADVANCED EXAMPLE  3 - Ethernet with UDP packets
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_vic.h"
#include "91x_wiu.h"
#include "91x_ssp.h"
#include <stdint.h>
#include "n3310.h"
#include "91x_enet.h"
#include "debug.h"
#include "uip.h"
#include "uip_arp.h"
#include "timer.h"
#include "armint.h"



#define BUF ((struct uip_eth_hdr *)&uip_buf[0])

#ifndef NULL
#define NULL (void *)0
#endif /* NULL */



/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG secure)
    delay(1000000);

    //Ustawienie petli PLL
    setup_pll();

    //Initialize debug
    debug_init();

    //Initialize system clock
    clock_init();
    
    //Initialize ENET library
    ENET_InitClocksGPIO();

    //Initialize ENET
    ENET_Init(PHY_HALFDUPLEX_10M);

    //Start receive and transmit
    ENET_Start();

    //Enable IRQ interrupts
    enable_irq();
    
    // ------------ uIP Init ---------------------
      int i;
      uip_ipaddr_t ipaddr;
      struct timer periodic_timer, arp_timer;

      timer_set(&periodic_timer, CLOCK_SECOND / 2);
      timer_set(&arp_timer, CLOCK_SECOND * 10);
      
      uip_init();
      
     /*****      KONFIGURACJA ADRESU  IP **********
     * w ponizszych polach nalezy wpisac:
     * adres ip, brame domyslna, 
     * oraz maske podsieci    
     *********************************************/
      uip_ipaddr(ipaddr, 192,168,16,222);
      uip_sethostaddr(ipaddr);
      uip_ipaddr(ipaddr, 192,168,16,1);
      uip_setdraddr(ipaddr);
      uip_ipaddr(ipaddr, 255,255,255,0);
      uip_setnetmask(ipaddr);

      /* httpd_init(); */
      
        telnetd_init();
      
       /* hello_world_init(); */

      /*  {
          u8_t mac[6] = {1,2,3,4,5,6};
          dhcpc_init(&mac, 6);
          }*/
      
      /*uip_ipaddr(ipaddr, 127,0,0,1);
      smtp_configure("localhost", ipaddr);
      SMTP_SEND("adam@sics.se", NULL, "uip-testing@example.com",
    	    "Testing SMTP from uIP",
    	    "Test message sent by uIP\r\n");*/

      /*
        webclient_init();
        resolv_init();
        uip_ipaddr(ipaddr, 195,54,122,204);
        resolv_conf(ipaddr);
        resolv_query("www.sics.se");*/

    
      // ------------ uIP End ---------------------      
        while(1) 
        {
            uip_len = ENET_HandleRxPkt(uip_buf);
            if(uip_len > 0) 
            {
              if(BUF->type == htons(UIP_ETHTYPE_IP)) 
              {
            	  uip_arp_ipin();
            	  uip_input();
            	  /* If the above function invocation resulted in data that
        	   should be sent out on the network, the global variable
        	   uip_len is set to a value > 0. */
            	  if(uip_len > 0) 
            	  {
            		  uip_arp_out();
            		  ENET_TxPkt(uip_buf,uip_len);
            	  }
              	} 
              	else if(BUF->type == htons(UIP_ETHTYPE_ARP)) 
              	{
              		uip_arp_arpin();
              		/* If the above function invocation resulted in data that
        	   		should be sent out on the network, the global variable
        	   		uip_len is set to a value > 0. */
	              	if(uip_len > 0) 
	              	{
	              		ENET_TxPkt(uip_buf,uip_len);
	              	}
                }

            } 
            else if(timer_expired(&periodic_timer)) 
            {
              timer_reset(&periodic_timer);
              for(i = 0; i < UIP_CONNS; i++) 
              {
            	  uip_periodic(i);
            	  /* If the above function invocation resulted in data that
	        	   should be sent out on the network, the global variable
	        	   uip_len is set to a value > 0. */
            	  if(uip_len > 0) 
            	  {
            		 uip_arp_out();
            		 ENET_TxPkt(uip_buf,uip_len);
            	  }
              }

        #if UIP_UDP
              for(i = 0; i < UIP_UDP_CONNS; i++) 
              {
            	  uip_udp_periodic(i);
		        	/* If the above function invocation resulted in data that
		        	   should be sent out on the network, the global variable
		        	   uip_len is set to a value > 0. */
		        	if(uip_len > 0) 
		        	{
		        	  uip_arp_out();
		        	  ENET_TxPkt(uip_buf,uip_len);
		        	}
		       }
        #endif /* UIP_UDP */
              
              /* Call the ARP timer function every 10 seconds. */
              if(timer_expired(&arp_timer)) 
              {
            	  timer_reset(&arp_timer);
            	  uip_arp_timer();
              }
            }
          }

    return 0;
}



void uip_log(char *m)
{
  printf("uIP log message: %s\n", m);
}


