/*
    Str912 book example8 - Simple LCD character display in C source
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_vic.h"
#include "91x_wiu.h"
#include "91x_ssp.h"
#include <stdint.h>
#include "n3310.h"

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 3
    SCU->PRR1 |= __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 3
    SCU->PCGR1 |= __GPIO3;

    //Caly port 3 jako wejscie
    SCU->GPIOIN[3] = 0x00;

    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;

}

/*----------------------------------------------------------------------*/
#include "linux.xpm"
#include "kaczka.xpm"
/*----------------------------------------------------------------------*/
//Wcisnieto klawisz 1
void key1_pressed(void)
{
   lcd_clear();
   lcd_string("STR912 book",0,0);
   lcd_string("(c) Bryndza L.",0,1);
   lcd_string("Warsaw 2008",0,2);
   lcd_string("3310 Display",0,3);
   lcd_string("BoFF Corp",0,4);
   lcd_string("www.boff.pl",0,5);
}

/*----------------------------------------------------------------------*/
//Wcisnieto klawisz 2
void key2_pressed(void)
{
    lcd_clear();
    lcd_pixmap(linux_xpm);
}

/*----------------------------------------------------------------------*/
//Wcisnieto klawisz 2
void key3_pressed(void)
{
    lcd_clear();
    //Pasek o poczatu 10 szerokosci 10 wysokosci 20
    lcd_bar(0,10,20);
    //Pasek o poczatu 12 szerokosci 10 wysokosci 30
    lcd_bar(12,10,30);
    //Pasek o poczatu 24 szerokosci 10 wysokosci 45
    lcd_bar(24,10,45);
}

/*----------------------------------------------------------------------*/

//Wcisnieto klawisz 2
void key4_pressed(void)
{
    lcd_clear();
    lcd_pixmap(kaczka_xpm);
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG secure)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Init nokia graphics LCD
    lcd_init();

    uint8_t pkey=0xf0;
    uint8_t key;
    //Ekran powitalny
    lcd_string("Nacisnij dowolny klawisz (K1-K4)",0,0);

    while(1)
    {
        key = GPIO3->DR[0xf0<<2];
        if( (~key & pkey) & 0x10)
        {
            key1_pressed();
        }
        else if( (~key & pkey) & 0x20)
        {
            key2_pressed();
        }
        else if( (~key & pkey) & 0x40)
        {
            key3_pressed();
        }
        else if( (~key & pkey) & 0x80)
        {
            key4_pressed();
        }

        pkey = key;
    }
    //lcd_string("KUPA",0,0);
    //lcd_string("zenek",0,0);
    //lcd_pixmap(linux_xpm);
        return 0;
}



