/*
    Str912 book example12a - I2C library
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/*----------------------------------------------------------------------*/
/* Ustawienia GPIO
   P1.4 (CLK) - Ustawienia: ALTERNATE INPUT:  1 ALTERNATE OUTPUT: 3
   P1.6 (DAT) - Ustawienia: ALTERNATE INPUT:  1 ALTERNATE OUTPUT: 3 */

/*----------------------------------------------------------------------*/

#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_vic.h"
#include "91x_wiu.h"
#include "91x_ssp.h"
#include "91x_i2c.h"
#include <stdint.h>
#include "i2c.h"

/*----------------------------------------------------------------------*/

/* I2C IT enable */
#define  I2C_IT_Enable     0x01
#define  I2C_IT_Disable    0xFE

/* I2C Peripheral Enable/Disable */
#define  I2C_PE_Set        0x20
#define  I2C_PE_Reset      0xDF

/* Address direction bit */
#define I2C_ADD0_Set       0x01
#define I2C_ADD0_Reset     0xFE

/* I2C START Enable/Disable */
#define  I2C_Start_Enable       0x08
#define  I2C_Start_Disable      0xF7

/* I2C STOP Enable/Disable */
#define  I2C_Stop_Enable        0x02
#define  I2C_Stop_Disable       0xFD

/* I2C Masks */
#define  I2C_Frequency_Mask     0x1F
#define  I2C_AddressHigh_Mask   0xF9
#define  I2C_OwnAddress_Mask    0x0300
#define  I2C_StandardMode_Mask  0x7f
#define  I2C_FastMode_Mask      0x80
#define  I2C_Event_Mask         0x3FFF
#define  I2C_HeaderSet_Mask     0xF1
#define  I2C_HeaderReset_Mask   0xFE


/*----------------------------------------------------------------------*/

//Predkosc I2c - 100khz dla 48MHz
#define I2C_SPEED 250

/*----------------------------------------------------------------------*/
//Pobiera aktualny status na magistrali I2C
static uint16_t i2c_getevent(void)
{
  u16 flag1=0, flag2 =0, event=0;
  flag2 = I2C0->SR1;
  flag1 = I2C0->SR2;
  flag1 = flag1<<8;
  /* Get the last event value from I2C status register */
  event = ((( flag2 | flag1) & I2C_Event_Mask));
  /* Return the last event */
  return event;
}

/*----------------------------------------------------------------------*/
//Cala dlugosc oczekiwanego lancucha
static volatile int i2c_len;

//Aktualna ilosc bajtow wyslanych
static volatile int i2c_pos;

//Wskaznik na bufor
static uint8_t *i2c_buf;

//Biezacy status I2c
static volatile uint8_t i2c_stat = I2CSTAT_IDLE;

//Biezacy adres i2c
static volatile uint8_t i2c_addr;

//Uchwyt do zdarzenia odczytu
void (*pfn_read)(void);

//Uchwyt do zdarzenia odczytu
void (*pfn_write)(void);

/*----------------------------------------------------------------------*/
//I2C interrupt handler
static void i2c0_irq_handler(void) __attribute__ ((interrupt("IRQ")));
static void i2c0_irq_handler(void)
{
    switch(i2c_getevent())
    {
    case I2C_EVENT_MASTER_MODE_SELECT:		//EV5
	    I2C0->DR = i2c_addr;
	    break;
    case I2C_EVENT_MASTER_MODE_SELECTED:	//EV6
	    I2C0->CR |= 0x20;			//Clear EV6 (write cr)
	    if(i2c_stat==I2CSTAT_TX) I2C0->DR = i2c_buf[i2c_pos++];
	break;
    case I2C_EVENT_MASTER_BYTE_TRANSMITTED:	//EV8
	    if(i2c_pos==i2c_len)
	    {
	        I2C0->CR |= I2C_Stop_Enable;
	        i2c_stat = I2CSTAT_IDLE;
            if(pfn_write) pfn_write();
	    }
	    else
	    {
	        I2C0->DR = i2c_buf[i2c_pos++];
	    }
	    break;
    case I2C_EVENT_MASTER_BYTE_RECEIVED:	//EV7
	    if(i2c_pos==i2c_len-2)
	    {
	        I2C0->CR &= I2C_Ack_Disable;
	    }
	    if(i2c_pos==i2c_len-1)
	    {
	        I2C0->CR |= I2C_Stop_Enable;
	        i2c_stat = I2CSTAT_IDLE;
            if(pfn_read) pfn_read();
        }
        i2c_buf[i2c_pos++] = I2C0->DR;
	    break;
    default:
	    i2c_stat = I2CSTAT_ERR;
	    break;
    }
    //Informacja dla VIC (koniec przerwania)
    VIC1->VAR = 0;
}

/*----------------------------------------------------------------------*/

#define VIC1_I2C0_BIT 3
#define VIC1_I2C0 (1<<3)
#define VIC_SLOT_EN (1<<5)

/*----------------------------------------------------------------------*/
void i2c_init(int prio)
{
    //Odblokuj sygnal zegarowy
    SCU->PCGR1 |= __I2C0 | __GPIO1;
    //Odblokuj sygnal resetu
    SCU->PRR1 |= __I2C0 | __GPIO1;

    SCU->PRR1 &= ~__I2C0;
    SCU->PRR1 |= __I2C0;

    //Odblokowanie sygnalu zegarowego VIC
    SCU->PCGR0 |= __VIC;
    //Odblokowanie resetu VIC
    SCU->PRR0 |= __VIC;

    //P2.4 i P2.6 Alt out 3
    SCU->GPIOOUT[1] |= 3<<(2*4) | 3<<(2*6);
    //P2.4 i P2.6 Input
    GPIO1->DDR &= ~((1<<4) | (1<<6));
    //P2.4 i P2.6 Alt input 1
    SCU->GPIOIN[1] |=  (1<<4) | (1<<6);
    //P2.4 i P2.6 Open collector
    SCU->GPIOTYPE[1] |= (1<<4) | (1<<6);

    //Konfiguracja predkosci PCLK > 40MHz
    I2C0->OAR2 = 4<<5;

    //Wlaczenie modulu I2C
    I2C0->CR |= I2C_PE_Set;
    I2C0->CR |= I2C_PE_Set;

    //General call disable
    I2C0->CR &= I2C_GeneralCall_Disable;

    //ACK enable
    I2C0->CR |= I2C_Ack_Enable;


    //Ustawienie predkosci I2C (100kHz)
    I2C0->CCR = I2C_SPEED & 0x7f;
    I2C0->ECCR = I2C_SPEED >> 7;

    //Wlacz przerwania w I2c
    I2C0->CR |= I2C_IT_Enable;

    //Do wektora podstaw adres procedury obslugi
    VIC1->VAiR[prio] = (unsigned long)i2c0_irq_handler;
    //Ustaw slot na przerwanie od WIUOC
    VIC1->VCiR[prio] = VIC1_I2C0_BIT | VIC_SLOT_EN;
    //Zakwalifikuj przerwanie WIU jako IRQ
    VIC1->INTSR &= ~VIC1_I2C0;
    //Vic interrupt enable register
    VIC1->INTER = VIC1_I2C0;

}

/*----------------------------------------------------------------------*/
//Zapisuje sekwencje bajtow
int i2c_write_start(uint8_t addr,uint8_t *bytes,int size)
{
    //I2C sprawdz czy kontroler wolny
    if(i2c_stat != I2CSTAT_IDLE && i2c_stat!=I2CSTAT_ERR) return i2c_stat;
    i2c_buf = bytes;
    i2c_len = size;
    i2c_pos = 0;
    i2c_addr = addr;
    i2c_addr &= I2C_ADD0_Reset;
    i2c_stat = I2CSTAT_TX;
    //Wyslij start
    I2C0->CR |= I2C_Start_Enable;
    return 0;
}

/*----------------------------------------------------------------------*/
//Odczytuje sekwencje bajtow
int i2c_read_start(uint8_t addr,uint8_t *bytes,int size)
{
    //I2C Czy kontroler i2c jest wolny
    if(i2c_stat != I2CSTAT_IDLE && i2c_stat!=I2CSTAT_ERR) return i2c_stat;
    i2c_buf = bytes;
    i2c_len = size;
    i2c_pos = 0;
    i2c_addr = addr;
    i2c_addr |= I2C_ADD0_Set;
    i2c_stat = I2CSTAT_RX;
    if(size<2)
    {
	    I2C0->CR &= I2C_Ack_Disable;
    }
    else
    {
	    I2C0->CR |= I2C_Ack_Enable;
    }
    //Wyslij start
    I2C0->CR |= I2C_Start_Enable;
    return 0;
}

/*----------------------------------------------------------------------*/
//Sprawdz biezacy stan
int i2c_poll(void)
{
    return i2c_stat;
}

/*----------------------------------------------------------------------*/
//Funkcja wywolywana z kontekstu przerwania na zakonczenie odczytu
void i2c_register_read_handler(void (*pfn_read_handler)(void))
{
    pfn_read = pfn_read_handler;
}
/*----------------------------------------------------------------------*/
//Funkcja wywolywana z kontekstu przerwania na zakonczenie zapisu
void i2c_register_write_handler(void (*pfn_write_handler)(void))
{
    pfn_write = pfn_write_handler;
}

/*----------------------------------------------------------------------*/

