/*
    Str912 book example12a - I2C RTC
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_vic.h"
#include "91x_wiu.h"
#include "91x_ssp.h"
#include <stdint.h>
#include "n3310.h"
#include "armint.h"
#include "i2c.h"
/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 3
    SCU->PRR1 |= __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 3
    SCU->PCGR1 |= __GPIO3;

    //Caly port 3 jako wejscie
    SCU->GPIOIN[3] = 0x00;

    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;

}


/*----------------------------------------------------------------------*/
//Adres I2C dla RTC
#define I2C_RTC_ADDR 208

//Bufor na dane I2C
uint8_t reg_buf[16];

/*----------------------------------------------------------------------*/
//Adres zostal wyslany
static void addr_send_event(void)
{
    //Rozpocznij odczyt wszystkich rejestrow RTC
    i2c_read_start(I2C_RTC_ADDR,reg_buf,7);
}

/*----------------------------------------------------------------------*/
static void print_rtc(uint8_t *rtc)
{
    //Godziny
    lcd_uint(rtc[2]&0x3f,16,2,0,0);
    lcd_string(":",2,0);
    //Minuty
    lcd_uint(rtc[1]&0x7f,16,2,3,0);
    lcd_string(":",5,0);
    //Sekundy
    lcd_uint(rtc[0]&0x7f,16,2,6,0);
    lcd_string(":",5,0);
    //Rok
    lcd_uint(rtc[6],16,2,0,1);
    lcd_string("-",2,1);
    //Miesiac
    lcd_uint(rtc[5]&0x1f,16,2,3,1);
    lcd_string("-",5,1);
    //Dzien
    lcd_uint(rtc[4]&0x3f,16,2,6,1);
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG secure)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Init nokia graphics LCD
    lcd_init();

    //Inicjalizacja biblioteki I2C
    i2c_init(0);

    //Wlacz przerwania
    enable_irq();

    //Poczatkowe ustawienia zegara
    reg_buf[0] = 0;     //Adres
    reg_buf[1] = 0x00;  //Sekundy
    reg_buf[2] = 0x34;  //Minuty
    reg_buf[3] = 0x14;  //Godziny
    reg_buf[4] = 0x01;  //Dzien tyg
    reg_buf[5] = 0x12;  //Dzien
    reg_buf[6] = 0x03;  //Miesiac
    reg_buf[7] = 0x07;  //Rok

    //Zapisz start
    i2c_write_start(I2C_RTC_ADDR,reg_buf,8);

    //Czekaj na zakonczenie
    while( i2c_poll() & I2CSTAT_BUSY );

    //Rejestruj funkcje wywolywana po zakonczeniu zapisu
    i2c_register_write_handler(addr_send_event);

    while(1)
    {
	//Rozpocznij zapis
	reg_buf[0] = 0;		//Adres pierwszego rejestru
	i2c_write_start(I2C_RTC_ADDR,reg_buf,1);
	//Czekaj na zakonczenie transmisji
	while( i2c_poll() & I2CSTAT_BUSY );
	//Poczekaj chwile
	delay(100000);
	//Wypisz godzine
	print_rtc(reg_buf);
    }
    return 0;
}



