/*
    Str912 book example12 - NOKIA 3310 LCD
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/* Inicjalizacja wyswietlacza LCD */

#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_vic.h"
#include "91x_wiu.h"
#include "91x_ssp.h"
#include <stdint.h>
#include <string.h>
#include <stdlib.h>

/*----------------------------------------------------------------------*/
/* Przypisania wyswietlacza LCD N3310 (84x48) do pinow mikrokontrolera
 * SCLK - P2.4 (SCLK) (typ Alternate Output 2)
 * SDIN - P2.5 (MOSI) (typ Alternate Output 2)
 * DC   - P2.0 (Alternate Output 1)
 * SEL  - P2.1 (Alternate Output 1)
 * RES - P2.3  (Alternate Output 1)
 * Urzadzenie dolaczone do SSP0
 */
/*----------------------------------------------------------------------*/

//Port uzyty do sterowania wyswietlaczem
#define LCD_PORT 2
//Bit SCLK (Sprzetowy SPI)
#define SCLK_PIN 4
//Bit SDIN (Sprzetowy SPI)
#define SDIN_PIN 5
//BIT DC (sterowanie programowe)
#define DC_PIN 0
//Bit SEL (sterowanie programowe)
#define SEL_PIN 1
//Bit RES - reset (sterowanie programowe)
#define RES_PIN 3


/*----------------------------------------------------------------------*/
/* Makrodefinicje obslugi GPIO */

/* Tryby pracy portu GPIO */
#define GPIOOUT_IN 0
#define GPIOOUT_ALT1 1
#define GPIOOUT_ALT2 2
#define GPIOOUT_ALT 3


#define XGPIO(X) (GPIO##X)
#define GPIO(X) XGPIO(X)

//Konfiguracja GPIOOUT
#define GPIOOUT_CFG(port,pin,func) do { SCU->GPIOOUT[(port)] &= ~(3<<(2*(pin))); \
                                        SCU->GPIOOUT[(port)] |= (func)<<(2*(pin));    } while(0)

//Konfiguracja DDR
#define DDR_OUT 1
#define DDR_IN 0
#define DDR_CFG(port,pin,out) if(out) XGPIO(port)->DDR |= 1<<(pin); else XGPIO(port)->DDR &= ~(1<<(pin))


//Zmiana stanu linii sterowanych programowo
#define DC(x) GPIO(LCD_PORT)->DR[(1<<DC_PIN)<<2] = (x)<<DC_PIN
#define SEL(x) GPIO(LCD_PORT)->DR[(1<<SEL_PIN)<<2] = (x)<<SEL_PIN
#define RES(x) GPIO(LCD_PORT)->DR[(1<<RES_PIN)<<2] = (x)<<RES_PIN

#define SSP_ENABLE (1<<1)
#define SSP_BUSY (1<<4)

//Fonts pattern
static const char cg_rom[];

/*----------------------------------------------------------------------*/
//Hardware initialization Init
static void hw_init()
{
    //Wylaczamy reset
    SCU->PRR1 |= __GPIO2;

    //Wlaczamy sygnal zegarowy
    SCU->PCGR1 |= __GPIO2;

    //Enable clocks for SSP0
    SCU->PRR1 |= __SSP0;

    //Disable reset for SSP0
    SCU->PCGR1 |= __SSP0;

    //Konfiguracja GPIO
    GPIOOUT_CFG(LCD_PORT,SCLK_PIN,GPIOOUT_ALT2);      // P2.4
    GPIOOUT_CFG(LCD_PORT,SDIN_PIN,GPIOOUT_ALT2);      // P2.5
    GPIOOUT_CFG(LCD_PORT,DC_PIN,GPIOOUT_ALT1);        // P2.0
    GPIOOUT_CFG(LCD_PORT,SEL_PIN,GPIOOUT_ALT1);       // P2.1
    GPIOOUT_CFG(LCD_PORT,RES_PIN,GPIOOUT_ALT1);

    //Kierunek wyjscie dla linii obslugiwanych programowo
    DDR_CFG(LCD_PORT,DC_PIN,DDR_OUT);
    DDR_CFG(LCD_PORT,SEL_PIN,DDR_OUT);
    DDR_CFG(LCD_PORT,RES_PIN,DDR_OUT);

    //Linia DC stan 1
    DC(1);
    //Linia SEL stan 1
	SEL(1);
    //Linia RES stan 1
    RES(1);
    //Konfiguracja SPI
    SSP0->CR0 = SSP_DataSize_8b;
    //Ustaw preskaler na 24/8 = 3MHz
    SSP0->PR = 8;
    //Wlacz SPI
    SSP0->CR1 = SSP_ENABLE;

}
/*----------------------------------------------------------------------*/
//Write SPI
static inline void hw_spi_write(uint8_t byte)
{
    SSP0->DR = byte;
    while(SSP0->SR & SSP_BUSY);
}

/*----------------------------------------------------------------------*/
//Initialize LCD display
void lcd_init(void)
{
    //Setup microcontroler hardware
    hw_init();

    //Reset wyswietlacza
    RES(0);
    for(int i=0;i<1000000;i++) asm volatile("nop");
    RES(1);

    //Inicjalizacja
    SEL(0);                 //Aktywuj sygnal
    DC(0);                  // INSTRUKCJE
	hw_spi_write(0x21);     //komendy rosrzerzone
	hw_spi_write(0xc8);     //Ustawienie Vop
	hw_spi_write(0x06);     //korekcja tempertury
	hw_spi_write(0x13);     //wspolczynnik multipleksowania
	hw_spi_write(0x20);     //komendy standardowe - adresowanie poziome
	hw_spi_write(0x0c);     //tryb wyswietlania stndard mode
	hw_spi_write(0x40);     //zerowanie licznika wierszy
	hw_spi_write(0x80);     //zerowanie licznika kolumn
    DC(1);                  //DANE
    SEL(1);                 //Deaktywuj sygnal
    SEL(0);                 //Aktywuj sygnal

    //Zerowanie pamieci ram wyswietlacza
    for(int i=0;i<504;i++)
	    hw_spi_write(0x0);
	//Deactivate select signal
    SEL(1);
}

/*----------------------------------------------------------------------*/
//Write character at current position
void lcd_char(char code)
{
    int code_point;
	//Dane
    DC(1);
	//Wyznacz wskaznik w tablicy bajtow
    code_point=((int)code*6);
	//Rozpocznij zapis fontow
	SEL(0);
    hw_spi_write(cg_rom[code_point++]);
	hw_spi_write(cg_rom[code_point++]);
	hw_spi_write(cg_rom[code_point++]);
	hw_spi_write(cg_rom[code_point++]);
	hw_spi_write(cg_rom[code_point++]);
	hw_spi_write(cg_rom[code_point]);
    //Koniec zapisu
    SEL(1);
}

/*----------------------------------------------------------------------*/
//ustawia pozycje wyswietlania (14 kolumn, 6 wierszy)
void lcd_pos(char x, char y)
{
	x=x*6;
	//Aktywacja zapisu
	SEL(0);
    //Instrukcje
    DC(0);
	hw_spi_write(y|0x40);       //zerowanie licznika wierszy
	hw_spi_write(x|0x80);       //zerowanie licznika kolumn
	// Deaktywacja zapisu
    SEL(1);
    DC(1);
}

/*----------------------------------------------------------------------*/
//wyswietla lancuch na okreslonej pozycji
void lcd_string(const char *napis,char x,char y)
{
	//Pozycja tekstu na wyswietlaczu
    lcd_pos(x,y);
	DC(1);
    while(*napis) lcd_char(*napis++);
}

/*-------------------------------------------------------------*/
//Wyswietl liczbe ull
void lcd_uint(unsigned int num,int radix,int zeros,char x,char y)
{
    static const char hexdigit[] = {'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'};
    char buf[10];
    unsigned int calk;
    int i=0;
    calk = num;
    while(calk)
    {
      calk /= radix;
      i++;
    }
    if(zeros) if(i<zeros) i=zeros;
    if(i)
    {
        calk = num;
        buf[i] = 0;
        for(--i;i>=0;i--)
        {
           buf[i] = hexdigit[calk % radix];
           calk /= radix;
        }
    }
    else
    {
        buf[0] = '0';
        buf[1] = 0;
    }
    lcd_string(buf,x,y);
}
/*----------------------------------------------------------------------*/
//Clear LCD display
void lcd_clear(void)
{
    SEL(0);                 //Aktywuj sygnal
    //Zerowanie pamieci ram wyswietlacza
    for(int i=0;i<504;i++)
	    hw_spi_write(0x0);
	//Deactivate select signal
    SEL(1);
}

/*----------------------------------------------------------------------*/
int lcd_pixmap(const char *xpm[])
{
    char lbuf[32];
    int x=0,y=0,shift=0;
    char ch_on=0;
    SEL(0);
    DC(0);                  //Instrukcje
    hw_spi_write(0x40);     //Zerowanie licznika wierszy
    hw_spi_write(0x80);     //Zerowanie licznika kolumn
    hw_spi_write(0x22);     //Tryp pionowy
    SEL(1);
    DC(1);                  //Teraz rozkazy

    strncpy(lbuf,xpm[0],sizeof(lbuf));
    lbuf[sizeof(lbuf)-1]  =  0;

    char *pch=strtok(lbuf," ");
    for(int i=0 ;pch!=NULL; i++,pch=strtok(NULL," "))
    {
        switch(i)
       {
           case 0:
               x = atoi(pch);
               break;
           case 1:
               y = atoi(pch);
               break;
           case 2:
               shift = atoi(pch);
               break;
           case 3:
               if(atoi(pch)!=1) return -2;
               break;
           default:
               break;
       }
    }
    if(x>84) x = 84;
    if(y%8 != 0) y+= y%8;
    if(y>48) y = 48;
    for(int i=0;i<=shift;i++)
    {
        if(strstr(xpm[i],"#000000")!=NULL)
        {
            ch_on = xpm[i][0];
            break;
        }
    }
    if(ch_on==0) return -1;
    uint8_t sbyte=0;
    DC(1);      //Dane
    SEL(0);     //Aktywacja
    for(int j=0;j<x;j++)
    for(int i=0;i<y;i++)
    {
        if(xpm[i+shift+1][j] == ch_on)
        {
            sbyte |= 1<<(i%8);
        }
        else
        {
            sbyte &= ~(1<<(i%8));
        }
        if(i%8==7)
        {
            hw_spi_write(sbyte);
        }
    }
    SEL(1);

    //Przywrocenie trybu poczatkowego
    SEL(0);
    DC(0);                  //Instrukcje
    hw_spi_write(0x40);     //Zerowanie licznika wierszy
    hw_spi_write(0x80);     //Zerowanie licznika kolumn
    hw_spi_write(0x20);     //Tryp pionowy
    SEL(1);
    DC(1);                  //Teraz rozkazy
    return 0;
}

/*----------------------------------------------------------------------*/
//Wyswietla pionowy pasek o parametrach wysokosc i szerokosc
int lcd_bar(uint8_t x,uint8_t s,uint8_t h)
{
    if(s+x > 84) return -1;
    SEL(0);
    DC(0);                        //Instrukcje
    hw_spi_write(0x40);           //Zerowanie licznika wierszy
    hw_spi_write(0x80|x);         //Ustawianie licznika kolumn
    hw_spi_write(0x22);           //Tryp pionowy
    SEL(1);
    DC(1);                        //Teraz rozkazy
    h = 48 - h;
    uint8_t ff = h / 8;
    uint8_t nf = h % 8;
    uint8_t nb = 0;

    for(int i=0;i<nf;i++) nb |= 1<<i;
    nb = ~nb;

    SEL(0);
    DC(1);     //Dane
     //Petla glowna
    for(int i=0;i<s;i++)
    for(int j=0;j<6;j++)
    {
        if(j<ff) hw_spi_write(0x00);
        else if(j==ff) hw_spi_write(nb);
        else hw_spi_write(0xff);
    }
    //Koniec transmisji
    SEL(1);

    //Przywrocenie trybu poziomego
    SEL(0);
    DC(0);                  //Instrukcje
    hw_spi_write(0x40);     //Zerowanie licznika wierszy
    hw_spi_write(0x80);     //Zerowanie licznika kolumn
    hw_spi_write(0x20);     //Tryp pionowy
    SEL(1);
    DC(1);                  //Teraz rozkazy
    return 0;
}
/*----------------------------------------------------------------------*/
//Fonts pattern
static const char cg_rom[]=
{
    0x48,0x54,0x56,0x54,0x20,0,     //kod 
    0x20,0x54,0x54,0xd4,0x78,0,     //kod ¹
    0x38,0x44,0x46,0x44,0x20,0,     //kod æ
    0,0,0,0,0,0,                    //kod 3
    0,0,0,0,0,0,                    //kod 4
    0,0,0,0,0,0,                    //kod 5
    0,0,0,0,0,0,                    //kod 6
    0,0,0,0,0,0,                    //kod 7
    0,0,0,0,0,0,                    //kod 8
    0,0,0,0,0,0,                    //kod 9
    0,0,0,0,0,0,                    //kod A
    0,0,0,0,0,0,                    //kod B
    0,0,0,0,0,0,                    //kod 12
    0,0,0,0,0,0,                    //kod 13
    0,0,0,0,0,0,                    //kod 14
    0,0,0,0,0,0,                    //kod 15
    0,0,0,0,0,0,                    //kod 16
    0,0,0,0,0,0,                    //kod 17
    0,0,0,0,0,0,                    //kod 18
    0,0,0,0,0,0,                    //kod 19
    0,0,0,0,0,0,                    //kod 20
    0,6,9,9,6,0,                    //kod 21 stopnie C
    0x2a,0x1c,0x3e,0x1c,0x2a,0,     //kod 22 (sloneczko)
    0x24,0x46,0x46,0x3e,0x06,0x04,  //kod 23(parasolka)
    0,0xc0,0xe0,0xf0,0xc8,0x88,     //kod 24 (szpula magnet)
    0x88,0xc8,0xf0,0xe0,0xc0,0,     //kod 25
    0,0xc0,0x20,0x10,0x38,0xf8,     //kod 26
    0xf8,0x38,0x10,0x20,0xc0,0,     //kod 27
    0,3,4,8,0x1c,0x1f,              //kod 28
    0x1f,0x1c,0x08,4,3,0,           //kod 29
    0,3,7,0x0f,0x13,0x11,           //kod 30
    0x11,0x13,0x0f,7,3,0,           //kod 31
    0,0,0,0,0,0,                    //kod spacja
    0,0,0xf2,0,0,0,                 //kod !
    0,3,0,3,0,0,                    //kod "
    0x14,0x7f,0x14,0x7f,0x14,0,     //kod #
    0x24,0x2a,0x7f,0x2a,0x12,0,     //kod $
    0x23,0x13,8,0x64,0x62,0,        //kod %
    0x36,0x49,0x55,0x22,0x50,0,     //kod &
    0,5,3,0,0,0,                    //kod '
    0,0x1c,0x22,0x41,0,0,           //kod (
    0,0x41,0x22,0x1c,0,0,           //kod )
    0x14,8,0x3e,8,0x14,0,           //kod *
    8,8,0x3e,8,8,0,                 //kod +
    0,0x50,0x30,0,0,0,              //kod ,
    8,8,8,8,8,8,                    //kod -
    0,0x60,0x60,0,0,0,              //kod .
    0x20,0x10,8,4,2,0,              //kod /
    0x3e,0x51,0x49,0x45,0x3e,0,     //kod 0
    0,0x42,0x7f,0x40,0,0,           //kod 1
    0x42,0x61,0x51,0x49,0x46,0,     //kod 2
    0x21,0x41,0x45,0x4b,0x31,0,     //kod 3
    0x18,0x14,0x12,0x7f,0x10,0,     //kod 4
    0x27,0x45,0x45,0x45,0x39,0,     //kod 5
    0x3c,0x4a,0x49,0x49,0x30,0,     //kod 6
    1,0x71,9,5,3,0,                 //kod 7
    0x36,0x49,0x49,0x49,0x36,0,     //kod 8
    6,0x49,0x49,0x29,0x1e,0,        //kod 9
    0,0x36,0x36,0,0,0,              //kod :
    0,0x56,0x36,0,0,0,              //kod ;
    8,0x14,0x22,0x41,0,0,           //kod <
    0x14,0x14,0x14,0x14,0x14,0x00,  //kod=
    0,0x41,0x22,0x14,8,0,           //kod >
    2,1,0x51,9,6,0,                 //kod ?
    0x30,0x49,0x79,0x41,0x3e,0,     //kod @
    0x7E,0x11,0x11,0x11,0x7E,0x00,
    0x7F,0x49,0x49,0x49,0x36,0x00,
    0x3E,0x41,0x41,0x41,0x22,0x00,
    0x7F,0x41,0x41,0x22,0x1C,0x00,
    0x7F,0x49,0x49,0x49,0x41,0x00,
    0x7F,0x09,0x09,0x09,0x01,0x00,
    0x3E,0x41,0x49,0x49,0x7A,0x00,
    0x7F,0x08,0x08,0x08,0x7F,0x00,
    0x00,0x41,0x7F,0x41,0x00,0x00,
    0x20,0x40,0x41,0x3F,0x01,0x00,
    0x7F,0x08,0x14,0x22,0x41,0x00,
    0x7F,0x40,0x40,0x40,0x40,0x00,
    0x7F,0x02,0x0C,0x02,0x7F,0x00,
    0x7F,0x04,0x08,0x10,0x7F,0x00,
    0x3E,0x41,0x41,0x41,0x3E,0x00,
    0x7F,0x09,0x09,0x09,0x06,0x00,
    0x3E,0x41,0x51,0x21,0x5E,0x00,
    0x7F,0x09,0x19,0x29,0x46,0x00,
    0x46,0x49,0x49,0x49,0x31,0x00,
    0x01,0x01,0x7F,0x01,0x01,0x00,
    0x3F,0x40,0x40,0x40,0x3F,0x00,
    0x1F,0x20,0x40,0x20,0x1F,0x00,
    0x3F,0x40,0x38,0x40,0x3F,0x00,
    0x63,0x14,0x08,0x14,0x63,0x00,
    0x07,0x08,0x70,0x08,0x07,0x00,
    0x61,0x51,0x49,0x45,0x43,0x00,
    0x00,0x7F,0x41,0x41,0x00,0x00,
    0x02,0x04,0x08,0x10,0x20,0x00,
    0x00,0x41,0x41,0x7F,0x00,0x00,
    0x04,0x02,0x01,0x02,0x04,0x00,
    0x40,0x40,0x40,0x40,0x40,0x00,
    0x00,0x01,0x02,0x04,0x00,0x00,
    0x20,0x54,0x54,0x54,0x78,0x00,
    0x7F,0x48,0x44,0x44,0x38,0x00,
    0x38,0x44,0x44,0x44,0x20,0x00,
    0x38,0x44,0x44,0x48,0x7F,0x00,
    0x38,0x54,0x54,0x54,0x18,0x00,
    0x08,0x7E,0x09,0x01,0x02,0x00,
    0x0C,0x52,0x52,0x52,0x3E,0x00,
    0x7F,0x08,0x04,0x04,0x78,0x00,
    0x00,0x44,0x7D,0x40,0x00,0x00,
    0x20,0x40,0x44,0x3D,0x00,0x00,
    0x7F,0x10,0x28,0x44,0x00,0x00,
    0x00,0x41,0x7F,0x40,0x00,0x00,
    0x7C,0x04,0x18,0x04,0x78,0x00,
    0x7C,0x08,0x04,0x04,0x78,0x00,
    0x38,0x44,0x44,0x44,0x38,0x00,
    0x7C,0x14,0x14,0x14,0x08,0x00,
    0x08,0x14,0x14,0x18,0x7C,0x00,
    0x7C,0x08,0x04,0x04,0x08,0x00,
    0x48,0x54,0x54,0x54,0x20,0x00,
    0x04,0x3F,0x44,0x40,0x20,0x00,
    0x3C,0x40,0x40,0x20,0x7C,0x00,
    0x1C,0x20,0x40,0x20,0x1C,0x00,
    0x3C,0x40,0x30,0x40,0x3C,0x00,
    0x44,0x28,0x10,0x28,0x44,0x00,
    0x0C,0x50,0x50,0x50,0x3C,0x00,
    0x44,0x64,0x54,0x4C,0x44,0x00,
    0x00,0x08,0x36,0x41,0x00,0x00,
    0x00,0x00,0x7F,0x00,0x00,0x00,
    0x00,0x41,0x36,0x08,0x00,0x00,
    0x10,0x08,0x08,0x10,0x08,0x00,
    0x78,0x46,0x41,0x46,0x78,0x00,
};

/*----------------------------------------------------------------------*/

