/*
    Str912 book example14 - MMC card and SPI 
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//----- Include Files ---------------------------------------------------------
#include <stdint.h>
#include "mmc.h"
#include <91x_map.h>
#include <91x_scu.h>
#include <91x_ssp.h>


//------- Pin Defs ----------------------------------------------------------
/*
    Konfiguracja P5.4 - SCK Tryb: Alternate Output 2
    Konfiguracja P5.5 - MOSI  Tryb: Alternate Output 2
    Konfiguracja P5.6 - MISO Tryb: Alternate Input 1
    Konfiguracja P5.7 - CS (Sterowany programowo) Tryb: GPIO wyjscie
*/


//Maski dla pinow
#define P5_4_GPIOUT_MASK (3<<(2*4))
#define P5_5_GPIOUT_MASK (3<<(2*5))
#define P5_6_GPIOUT_MASK (3<<(2*6))
#define P5_7_GPIOUT_MASK (3<<(2*7))

//Ustawienia poszczegolnych pinow
#define P5_4_GPIOUT_ALT_OUT2 (2<<(2*4))
#define P5_5_GPIOUT_ALT_OUT2 (2<<(2*5))
#define P5_7_GPIOUT_ALT_OUT1 (1<<(2*7))
#define P5_6_GPIOIN_ALT_IN1 (1<<6)


#define P5_4_GPIOTYPE_OC (1<<4)
#define P5_5_GPIOTYPE_OC (1<<5)
#define P5_7_GPIOTYPE_OC (1<<7)

#define P5_4_DDR_OUT (1<<4)
#define P5_5_DDR_OUT (1<<5)
#define P5_7_DDR_OUT (1<<7)

#define CS1() GPIO5->DR[0x80<<2] = 0x80
#define CS0() GPIO5->DR[0x80<<2] = 0

//------------------- SPI functions ----------------------------------------
#define SSP_ENABLE (1<<1)
#define SSP_BUSY (1<<4)
//Inicjalizacja protokolu mmc i spi
void mmcInit(void)
{
	/**** Konfiguracja portow IO ******/
	//Odblokuj zegar portu 5 i SSP0
	SCU->PRR1 |= __GPIO5 | __SSP0;	
	//Odblokuj reset portu 5 i SSP0
	SCU->PCGR1 |= __GPIO5 | __SSP0;
	//Konfiguracja portow
	SCU->GPIOOUT[5] &= ~(P5_4_GPIOUT_MASK|P5_5_GPIOUT_MASK|P5_6_GPIOUT_MASK|P5_7_GPIOUT_MASK);
	SCU->GPIOOUT[5] |= P5_4_GPIOUT_ALT_OUT2|P5_5_GPIOUT_ALT_OUT2|P5_7_GPIOUT_ALT_OUT1;
	SCU->GPIOIN[5] |= P5_6_GPIOIN_ALT_IN1;
	SCU->GPIOTYPE[5] &= ~(P5_4_GPIOTYPE_OC|P5_5_GPIOTYPE_OC|P5_7_GPIOTYPE_OC);
	GPIO5->DDR |= P5_4_DDR_OUT|P5_5_DDR_OUT|P5_7_DDR_OUT;
	CS1();  //Chip select niektywny
	/**** Konfiguracja kontrolera SSP ******/
	//Set low speed and 8 bit size
	SSP0->CR0 = (255<<8)|SSP_DataSize_8b;
	//Ustaw preskaler na 24MHz
	SSP0->PR = 2;
	SSP0->CR1 = SSP_ENABLE;
}


//Nadaje i odbiera znak po SPI
static uint8_t spiTransferByte(uint8_t byte)
{
	SSP0->DR = byte;
	while(SSP0->SR & SSP_BUSY);		//Czekaj na koniec nadaw
	byte = SSP0->DR;
	return byte;
}

//Funkcja odczytujaca bajt w powietrze
static void flushByte(uint8_t count)
{
	while(count--)
	    spiTransferByte(0xFF);
}


//Reset reset i uruchimienie karty mmc
uint8_t mmcReset(void)
{
	uint16_t retry;
	uint8_t r1=0;

	retry = 0;
	do
	{
		// Zapisz 10 pustych bajtow
		flushByte(10); 
		// Zerowanie karty 
		r1 = mmcSendCommand(MMC_GO_IDLE_STATE, 0);
		// Licznik wznowien
		retry++;
		if(retry>50) return 0xff;
	} while(r1 != 0x01);

	retry = 0;
	do
	{
		// Inicjalizacja karty
		r1 = mmcSendCommand(MMC_SEND_OP_COND, 0);
		// Licznik wznowien
		retry++;
		if(retry>500) return 0xff;
	} while(r1);
		
	// Wylacz sprawdzenie CRC
	r1 = mmcSendCommand(MMC_CRC_ON_OFF, 0);
	// Ustaw rozmiar bloku na 512 bajtow
	r1 = mmcSendCommand(MMC_SET_BLOCKLEN, 512);
	//Teraz mozna zwiekszyc predkosc SPI z 400khz na max
	//Wylacz kontroler SPI
	SSP0->CR1 = 0;
	//Predkosc maksymalna
	SSP0->CR0 = (0<<8)|SSP_DataSize_8b;
	//Zalacz kontroler SPI
	SSP0->CR1 = SSP_ENABLE;
	// Wszystko ok
	return 0;
}

//Wysyla komende do karty MMC
uint8_t mmcSendCommand(uint8_t cmd, uint32_t arg)
{
	uint8_t r1;
	// Wlacz Chip Select
	CS0();
	// Wyslij komende
	r1 = mmcCommand(cmd, arg);
	// Zwolnij Chip Select
	CS1();
	return r1;
}

/* Odczytuje sektor o numerze sector do bufora buffer
 * z karty MMC zwraca 0 gdy OK
 */
uint8_t mmcRead(uint8_t* buffer,uint32_t sector)
{
	uint8_t r1;
	uint16_t i;
	// Wlacz CS
	CS0();
	// Wyslij komende odczytu bloku
	r1 = mmcCommand(MMC_READ_SINGLE_BLOCK, sector<<9);
	// Sprawdz odpowiedz
	if(r1 != 0x00)
	{
		CS1();
		return r1;
	}
	// Czekaj na gotowosc karty
	while(1)
	{
		r1 = spiTransferByte(0xFF);
		if( r1 == MMC_STARTBLOCK_READ) 
			break;
		if((r1&MMC_DE_CHECK_MASK)==MMC_DE_ERROR)
		{
			CS1();
			return r1;
		}
	}

	// Odczytuj dane
	for(i=0; i<0x200; i++)
	{
		*buffer++ = spiTransferByte(0xFF);
	}
	// Nie sprawdzaj CRC
	flushByte(2);
	// Zwolnij CS
	CS1();
	// Wszystko OK
	return 0;
}

/* Zapisuje dane z bufora buffer do sektora o numerze
 * sector zwraca 0 gdy wszystko w porzadku */
uint8_t mmcWrite(uint8_t* buffer,uint32_t sector)
{
	uint8_t r1;
	uint16_t i;
	// Uruchom CS
	CS0();
	// Wyslij komende odczytu
	r1 = mmcCommand(MMC_WRITE_BLOCK, sector<<9);
	// Sprawdz odpowiedz karty
	if(r1 != 0x00)
	{
		CS1();
		return r1;
	}
	// Wyslij pusty bajt
	spiTransferByte(0xFF);
	// Wyslij kod rozpoczecia transferu
	spiTransferByte(MMC_STARTBLOCK_WRITE);
	// Zapisz dane z bufora
	for(i=0; i<0x200; i++)
	{
		spiTransferByte(*buffer++);
	}
	// zapisz 16-bitowy CRC - nieistotny
	spiTransferByte(0xFF);
	spiTransferByte(0xFF);
	// Sprawdz token odpowiedzi
	r1 = spiTransferByte(0xFF);
	if( (r1&MMC_DR_MASK) != MMC_DR_ACCEPT)
	{
		CS1();
		return r1;
	}
	// Czekaj az karta bedzie wolna
	while(!spiTransferByte(0xFF));
	// Zwolnij CS
	CS1();
	// Zwroc OK
	return 0;
}

//Wysyla komende do SPI 
uint8_t mmcCommand(uint8_t cmd, uint32_t arg)
{
	uint8_t r1;
	uint8_t retry=0;
	// wyslij komende
	spiTransferByte(cmd | 0x40);
	spiTransferByte(arg>>24);
	spiTransferByte(arg>>16);
	spiTransferByte(arg>>8);
	spiTransferByte(arg);
	spiTransferByte(0x95);	
	// czekaj na odpowiedz
	// Po osmiu probach koniec - blad
	while((r1 = spiTransferByte(0xFF)) == 0xFF)
		if(retry++ > 8) break;
	// Zwroc odpwiedz
	return r1;
}


//Zwraca calkowita pojemnosc dysku mmc
uint64_t mmcCapacity(void)
{
	uint16_t size,blen;
	uint8_t mult,r1;
	// wlacz CS
	CS0();
	r1 = mmcCommand(MMC_SEND_CSD, 0);
	if(r1 != 0x00)
	{
		CS1();
		return 0;
	}
	uint16_t retry=0;
	while(1)
	{
		r1 = spiTransferByte(0xFF);
		if( r1 == MMC_STARTBLOCK_READ) 
			break;
		if((r1&MMC_DE_CHECK_MASK)==MMC_DE_ERROR)
		{
			CS1();
			return 0;
		}
		retry++;
		if(retry>500) 
		{
			CS1();
			return 0;
		}
	}
	flushByte(5);				// odrzuc pierwsze 6 bajtow rejestru CSD
	blen = spiTransferByte(0xff) & 0x0f;
	blen = 1<<blen;
	size = (spiTransferByte(0xff) & 3) << 10;	// najstarsze 2 bity C_SIZE
	size |= spiTransferByte(0xff) << 2;		// kolejne 8 bitow C_SIZE
	size |= (spiTransferByte(0xff) >> 6) & 3;	// najm-odsze 2 bity C_SIZE
	mult = (spiTransferByte(0xff) & 3) << 1;	// starsze 2 bity C_SIZE_MULT
	mult |= (spiTransferByte(0xff) >> 7) & 1;	// najm-odszy bit C_SIZE_MULT
	flushByte(7);				// odrzu reszt rejestru CSD i bajty CRC
	//Koniec transakcji MMC
	CS1();
	return ((uint64_t)(size+1)<<(mult+2))*blen;		// pojemnosc
}

