/*
    Str912 book example14 - MMC card and SPI 
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <91x_map.h>
#include <91x_scu.h>
#include <91x_fmi.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "uart.h"
#include "mmc.h"
#include "armint.h"

/*----------------------------------------------------------------------*/

#define USED_UART UART0

/*----------------------------------------------------------------------*/
// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
//Bufor odbiornika
unsigned char buf[600];

int bpos = 0;

/*----------------------------------------------------------------------*/
/* Funkcja glowna main */
int main(void)
{	
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie PLL
    setup_pll();

    //Inicjalizacja portu szeregowego
    uartInit(USED_UART,115200,15,128);
	
    //Zalacz przerwania
    enable_irq();
    
    //Napis powitalny
    printf("**********************************************************\n");
    printf("**** Ksiazka STR912 (c) Lucjan Bryndza Licence: GPLv3 ****\n");
    printf("************** Obsluga karty MMC za pomoca SPI ***********\n");
    printf("**********************************************************\n\n");
    
    
    //Inicjalizacja karty
    mmcInit();
    if(mmcReset()!=0) 
    {
	printf("MMC Init failed\n");
	return -1;
    }
    
    for(unsigned int sector=0;sector<3;sector++)
    {
	//Odczyt zerowego sektora
	if(mmcRead(buf,sector)!=0) 
	{
    	    printf("MMC read failed\n");
    	    return -1;
	}
	//Wyswietlenie zawartosci sektora ASCII
	printf("\nZawartosc sektora %d karty MMC\n",sector);
	uint8_t c;
	for(int i=0;i<512;i++)
	{
    	    c = buf[i];
    	    printf("%02x ",c);
	    if(!((i+1)%20)) printf("\n");
	}
    }
    return 0;
}

