/*
    Str912 book example15 - EEPROM library test
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <91x_map.h>
#include <91x_scu.h>
#include <91x_fmi.h>
#include "armint.h"
#include "uart.h"
#include <stdio.h>
#include <string.h>
#include <stdbool.h>
#include <stdlib.h>
#include "eeprom.h"
/*----------------------------------------------------------------------*/
#define USED_UART UART0

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}




/*----------------------------------------------------------------------*/

char in_buf[512];
uint16_t in_pos;
#define INBUF_SIZE 510

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie PLL
    setup_pll();

    //Inicjalizacja portu szeregowego
    uartInit(USED_UART,115200,15,128);
	
    //Zalacz przerwania
    enable_irq();

    //Napis powitalny
    printf("*****************STR912 Book *********************\n");
    printf("*** Autor: Lucjan Bryndza (c) 2007 Licence GPL ***\n");
    printf("*********** Emulacja pamieci EEPROM **************\n");
    printf("**************************************************\n");
    //Inicjalizacja pamieci FLASH
    int stat = eepromflaInit();
    //Jezeli nie udalo sie poprawnie zaincjalizowac
    if(stat<0)
    {
	    printf("Blad inicjalizacji pamieci flash (%d)\n",stat);
    }
    printf("Aby zapisac wartosc pod wskazany adres wpisz w adres,wartosc\n");
    printf("Aby odczytac wartosc z pod wskazanego adresu wpisz r adres,wartosc\n");

    //Petla glowna
    while(1)
    {
        int zn = uartReadByte(USED_UART);
        bool rw; uint16_t addr,data;
        int stat;
        if(zn!=UART_EMPTY)
        {
            uartWriteByte(USED_UART,zn);
            if(zn=='\r')
            {
                in_buf[in_pos] = 0;
                in_pos = 0;
                char *pos = strstr(in_buf,"r ");
                if(pos) rw = false;
                else
                {
                    pos = strstr(in_buf,"w ");
                    if(pos) rw = true;
                    else
                    {
                        printf("Blad nie chcesz ani zapisywac ani odczytywac\n");
                        continue;
                    }
                }
                if(rw==true)
                {
                    char *pos1 = strstr(pos,",");
                    if(pos1==NULL)
                    {
                        printf("Blad skladni !\n");
                        continue;
                    }
                    *pos1++ = 0;
                    data = atoi(pos1);
                }
                addr = atoi(pos+2);
                printf("Typ %d Adres: %d Dane: %d\n",(int)rw,(int)addr,(int)data);
                if(rw)
                {
                    stat = eepromflaWrite(addr,data);
                }
                else
                {
                    stat = eepromflaRead(addr);
                }
                printf("Status wykonania operacji %d\n",stat);
            }
            else
            {
                if(in_pos<INBUF_SIZE) in_buf[in_pos++] = zn;
            }
        }
    }
    return 0;
}

