/*
    Str912 book example16 - ADC without interrupt
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_adc.h"
#include "armint.h"
#include "uart.h"
#include <stdio.h>
/*----------------------------------------------------------------------*/
#define USED_UART UART0

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawianie przetwornika A/C w trybie pojedynczej konwersji */

#define ADC_ENABLE (1<<1)
#define ADC_STANDBY (1<<3)
#define ADC_CHANNEL_0 (0<<6)
#define ADC_CC0_NOWDT (3<<0)
#define ADC_START (1<<0)
#define ADC_ENDCONV (1<<15)

static void setup_adc(void)
{
	//Wlacz sygnal zegarowy dla GPIO4 i ADC
	SCU->PCGR1 |= __GPIO4 | __ADC;
	//Odblokuj sygnal resetu dla GPIO04 i ADC
	SCU->PRR1 |= __GPIO4 | __ADC;
	//Dolacz wejscie P4.0 do ADC
	SCU->GPIOIN[4] |= (1<<0);
	//Wlacz tryb analogowy linii P4.0
	SCU->GPIOANA |= (1<<0);
    //Ustaw preskaler czestoliwosc musi byc >25MHz
	ADC->PRS = 2; //(24MHz)
    //Enable ADC and select channel 0
	ADC->CR = ADC_ENABLE;
    ADC->CR |= ADC_CHANNEL_0;
	//Ustawienie bez watchdoga analogowego na P0.4
	ADC->CCR |= ADC_CC0_NOWDT;
}

/*----------------------------------------------------------------------*/
//Pobierz wartosc z przetwornika A/C

static uint16_t get_adc(void)
{
	//Rozpocznij przetwarzanie
	ADC->CR |= ADC_START;
	//Czekaj na koniec przetwarzania
	while(!(ADC->CR & ADC_ENDCONV));
	//Odczytaj wynik
	return ADC->DR0 & 0x3ff;
}


/*----------------------------------------------------------------------*/
#define V_REF 2.5

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie PLL
    setup_pll();

    //Inicjalizacja portu szeregowego
    uartInit(USED_UART,115200,15,256);

    //Zalacz przerwania
    enable_irq();

    //Inicjalizacja przetwornika A/C
    setup_adc();

    //Napis powitalny
    printf("*****************STR912 Book *********************\n");
    printf("*** Autor: Lucjan Bryndza (c) 2007 Licence GPL ***\n");
    printf("**********ADC bez przerwan - Woltomierz***********\n");
    printf("**************************************************\n");
    while(1)
    {
	int d_val = get_adc();
        float a_val = ((float)d_val / (float)0x3ff) * V_REF;
	printf("ADC=%d (0x%03x) U=%.3f V\n",d_val,d_val,a_val);
        delay(1000000);
    }
    return 0;
}

