/*
    Str912 book example18 - timer with interrupt
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include <stdint.h>
#include "armint.h"

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7 i 3
    SCU->PRR1 |= __GPIO7 | __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 7 i 3
    SCU->PCGR1 |= __GPIO7 | __GPIO3;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port 7 jako wyjsciowy
    GPIO7->DDR = 0xff;
    
    
    //Caly port 3 jako wejscie
    SCU->GPIOIN[3] = 0x00;
    
    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;
}

/*----------------------------------------------------------------------*/
//Tim01 select bit
#define SCU_TIM01SEL (1<<13)

//Bit wlaczenia przerwania od przepeln licznik
#define TIM_TOIE (1<<13)

//Bit zalaczenia timera
#define TIM_EN (1<<15)


//Bit flagi przepelnienia przerwania
#define TIM_TOF (1<<13)

//Prescaler 48M/7
#define TIM0_PRESCALE 7

//System ticks per second
#define HZ 105


//Bity kontrolera VIC
#define VIC0_TIM0 (1<<4)
#define VIC0_TIM0_BIT 4

//Zalaczenie slotu
#define VIC_SLOTEN (1<<5)


/*----------------------------------------------------------------------*/

//Zmienna odliczajaca
static volatile int tim;

//Przerwanie od ukladu czasowo licznikowego
static void irqTimHandler(void) __attribute__ ((interrupt("IRQ")));

void irqTimHandler(void)
{
    TIM0->SR &= ~TIM_TOF;
    if(tim) --tim;
    VIC0->VAR = 0;
}

/*----------------------------------------------------------------------*/


/* Ustawienie timera systemowego tak aby 100 razy na sekunde
 * zglaszal przerwanie */
static void setup_timer(void)
{
    /*** Ustawienie Timera 0 w tryb czasomierza ****/
    //Odblokowanie CLK i RST
    SCU->PCGR1 |= __TIM01;
    SCU->PRR1  |= __TIM01;
    //Licznik zlicza wewnetrzne impulsy
    SCU->CLKCNTR &= ~SCU_TIM01SEL;
    //Ustaw preskaler i przerwanie od przepelnienia
    TIM0->CR2 = TIM0_PRESCALE | TIM_TOIE;
    /*** Ustawienie kontrolera przerwan VIC ***/
    //Zalaczenie clocka dla VIC
    SCU->PCGR0 |= __VIC;
    //Reset
    SCU->PRR0 &= ~__VIC;
    SCU->PRR0 |= __VIC;
    //Do wektora podstaw adres procedury obslugi
    VIC0->VAiR[0] = (unsigned long)irqTimHandler;
    //Ustaw slot na przerwanie od WIUOC
    VIC0->VCiR[0] = VIC0_TIM0_BIT | VIC_SLOTEN;
    //Zakwalifikuj przerwanie jako IRQ
    VIC0->INTSR &= ~VIC0_TIM0;
    //Vic interrupt enable register
    VIC0->INTER = VIC0_TIM0;
    //Zalacz uklad czasowo licznikowy
    TIM0->CR1 = TIM_EN;
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Wlaczenie timera
    setup_timer();
    
    //Wlaczenie przerwan
    enable_irq();

    // petla nieskocznona
    while(1)
    {
	    int i;
	    for(i=0;i<4;i++)
	    {
        	// Zalacz wylacz ledy
		    GPIO7->DR[0x0f<<2]=(1<<i);
        	//Przepisz dane klawiszy na ledy
		    GPIO7->DR[0xf0<<2] = ~GPIO3->DR[0xf0<<2];
		    // wait...
        	tim = HZ/10;
        	while(tim);
    	}
    }
    return 0;
}

