/*
    Str912 book example1 - switch microcontroler to 96MHz
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"


/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawiajac PLL na duza czestotliwosc nalezy przestrzegac
 * maksymalnych dopuszczalnych czestotliwosci dla poszczegolnych
 * ukladow peryferyjnych oraz w przypadku taktowania pamieci
 * flash z czestotliwoscia 96MHz nalezy ustawic w kontrolerze
 * pamieci flash 2 cykle oczekiwania podczas dostepu do tej
 * pamieci, gdyz w przeciwnym przypadku procesor zawiesi sie.
 * Podczas majstrowania z czestotliwosciami zegarowymi dobrze
 * jest na samym poczatku funkcji main wstawic odpowiednia petle
 * opzniajaca aby mozna bylo zdarzyc uruchomic jtag w przypadku
 * blednego ustawienia czestotliwosci taktujacych, gdyz w przeciwnym
 * przypadku grozi to mozliwoscia zablokowania procesora i spowoduje
 * koniecznosc jego wymiany co moze byc przykre w skutkach zwlaszcza
 * dla kieszeni uzytkownika ;). */




/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7
    SCU->PRR1 |= __GPIO7;

    //Wlacz sygnal zegarowy dla portu 7
    SCU->PCGR1 |= __GPIO7;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port jako wyjsciowy
    GPIO7->DDR = 0xff;
}

/*----------------------------------------------------------------------*/
//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_gpio();

    // petla nieskocznona
    int loop_cnt = 0;
    while(1)
    {
	int i;
	for(i=0;i<8;i++)
	{
            // turn LEDs on/off
	    GPIO7->DR[0xff<<2]=(1<<i);
            // wait a while
	    delay(1000000);
        }
        if(loop_cnt<10)
        {
             loop_cnt++;
        }
        else if(loop_cnt==10)
        {
            //Za dziesiatym razem przyspiesz do 96MHz
            loop_cnt++;
            setup_pll();
        }
    }
    return 0;
}

