/*
    Str912 book example20 - timer1 PWM Mode / Waveform generation
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include <stdint.h>

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 6
    SCU->PRR1 |= __GPIO6;

    //Wlacz sygnal zegarowy dla portu 6
    SCU->PCGR1 |= __GPIO6;

    //Linia P6.0 Alternate output 2
    SCU->GPIOOUT[6]  = 0x0002;

}

/*----------------------------------------------------------------------*/

//Tim01 select bit
#define SCU_TIM01SEL (1<<13)

//Bit zalaczenia timera
#define TIM_EN (1<<15)

//Poziom sygnalu przy porownaniu
#define TIM_OLVL1 (1<<8)
#define TIM_OLVL2 (1<<9)

//Output compare pin enabled
#define TIM_OC1E (1<<6)

//Timer PWM bit
#define TIM_PWM (1<<4)





//Prescaler 48M/256
#define TIM0_PRESCALE 255


/* Ustawienie timera TIM0 do generacji sygnalu PWM */
static void setup_timer(void)
{
    /*** Ustawienie Timera 0 w tryb czasomierza ****/
    //Odblokowanie CLK i RST
    SCU->PCGR1 |= __TIM01;
    SCU->PRR1  |= __TIM01;
    //Licznik zlicza wewnetrzne impulsy
    SCU->CLKCNTR &= ~SCU_TIM01SEL;
    //Ustaw preskaler i przerwanie od przepelnienia
    TIM0->CR2 = TIM0_PRESCALE;
    //Period of signal
    TIM0->OC2R = 256;
    //PWM wave of form
    TIM0->OC1R = 100;
    //Zalacz uklad czasowo licznikowy
    TIM0->CR1 = TIM_EN | TIM_OLVL2 | TIM_OC1E | TIM_PWM;
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Wlaczenie timera
    setup_timer();

    // petla nieskocznona
    while(1);

    return 0;
}

