/*
    Str912 book example21 - input capture example
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include <stdint.h>
#include "armint.h"
#include "uart.h"

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 4
    SCU->PRR1 |= __GPIO4;

    //Wlacz sygnal zegarowy dla portu 4
    SCU->PCGR1 |= __GPIO4;

    //Pierwszy bit portu 4 wejscie alternatywne TIM0_ICAP1
    SCU->GPIOIN[4] = 0x01;

}

/*----------------------------------------------------------------------*/
//Tim01 select bit
#define SCU_TIM01SEL (1<<13)

//Bit wlaczenia przerwania od compare 1
#define TIM_OC1IE (1<<14)

//Bit wlaczenia przerwania od capture 1
#define TIM_IC1IE (1<<15)

//Bit zalaczenia timera
#define TIM_EN (1<<15)


//Compare flag 1
#define TIM_OCF1 (1<<14)

//Capture flag 1
#define TIM_ICF1 (1<<15)

//Prescaler 48M/47 = 1MHz
#define TIM0_PRESCALE 47

//Add timer counter
#define TIM0_RELOAD 10000


//Bity kontrolera VIC
#define VIC0_TIM0 (1<<4)
#define VIC0_TIM0_BIT 4

//Zalaczenie slotu
#define VIC_SLOTEN (1<<5)


/*----------------------------------------------------------------------*/

//Timer glowny
static volatile uint32_t tim;
//Timestamp zdarzenia
static volatile uint64_t event_tim;
//Flaga gotowosci
static volatile uint8_t event;

//Przerwanie od ukladu czasowo licznikowego
static void irqTimHandler(void) __attribute__ ((interrupt("IRQ")));

void irqTimHandler(void)
{
    if(TIM0->SR & TIM_OCF1)
    {
        //Output compare flag event..
        TIM0->OC1R = TIM0->CNTR + TIM0_RELOAD;
        tim++;
	    //Clear interrupt flag
	    TIM0->SR &= ~TIM_OCF1;
    }
    if(TIM0->SR & TIM_ICF1)
    {
        //Input compare flag event...
        event_tim = (uint64_t)TIM0->IC1R | (uint64_t)tim<<16;
	    event = 1;
	    //Clear interrupt flag
	    TIM0->SR &= ~TIM_ICF1;
    }
    //VIC end of interrupt signal
    VIC0->VAR = 0;
}

/*----------------------------------------------------------------------*/


/* Ustawienie timera systemowego tak aby 100 razy na sekunde
 * zglaszal przerwanie */
static void setup_timer(void)
{
    /*** Ustawienie Timera 0 w tryb czasomierza ****/
    //Odblokowanie CLK i RST
    SCU->PCGR1 |= __TIM01;
    SCU->PRR1  |= __TIM01;
    //Licznik zlicza wewnetrzne impulsy
    SCU->CLKCNTR &= ~SCU_TIM01SEL;
    //Ustaw preskaler i przerwanie od przepelnienia
    TIM0->CR2 = TIM0_PRESCALE | TIM_OC1IE | TIM_IC1IE;
    /*** Ustawienie kontrolera przerwan VIC ***/
    //Zalaczenie clocka dla VIC
    SCU->PCGR0 |= __VIC;
    //Reset
    SCU->PRR0 &= ~__VIC;
    SCU->PRR0 |= __VIC;
    //Do wektora podstaw adres procedury obslugi
    VIC0->VAiR[0] = (unsigned long)irqTimHandler;
    //Ustaw slot na przerwanie od WIUOC
    VIC0->VCiR[0] = VIC0_TIM0_BIT | VIC_SLOTEN;
    //Zakwalifikuj przerwanie jako IRQ
    VIC0->INTSR &= ~VIC0_TIM0;
    //Vic interrupt enable register
    VIC0->INTER = VIC0_TIM0;
    //Zalacz uklad czasowo licznikowy
    TIM0->CR1 = TIM_EN;
}

/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Wlaczenie timera
    setup_timer();

    //Inicjalizacja portu szeregowego
    uartInit(UART0,115200,0,2024);

    //Wlaczenie przerwan
    enable_irq();
    
    //Napis powitalny
    uartWriteString(UART0,"STR912 Book - Input capture example.\nCopyright L. Bryndza <lucjan.bryndza@ep.com.pl> GPL v3\n");
    uartWriteString(UART0,"Example trigger capture in falling edge\n");
    
    // petla nieskocznona
    while(1)
    {
	    if(event) 
	    {
	        disable_irq();
	        uint64_t n = event_tim;
	        enable_irq();
	        uartWriteString(UART0,"Event timestamp = ");
	        uartWriteUll(UART0,n);
	        uartWriteString(UART0,"\n");
	        event = 0;
	    }
    }
    return 0;
}

