/*
    Str912 book example22 - Watchdog example
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"


/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7 i 3
    SCU->PRR1 |= __GPIO7 | __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 7 i 3
    SCU->PCGR1 |= __GPIO7 | __GPIO3;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port 7 jako wyjsciowy
    GPIO7->DDR = 0xff;

    //Caly port 3 jako wejscie
    SCU->GPIOIN[3] = 0x00;

    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;
}
/*----------------------------------------------------------------------*/

//External couter enable bit
#define WDG_CR_EE (1<<2)

//Watchdog enable bit
#define WDG_CR_WE (1<<0)

//Watchdog magic seqences
#define WDG_MAGIC_KEY1 0xA55A
#define WDG_MAGIC_KEY2 0x5AA5


//Ustaw watchdog timer
static void setup_wdt(int timeout)
{
	//Odblokuj sygnal reset
	SCU->PRR1 |= __WDG; 
	//Odblokuj sygnal zegarowy
	SCU->PCGR1 |= __WDG;
	//Setup clock to 32768Hz crystal
	WDG->CR |= WDG_CR_EE;
	//Taktowanie 1024Hz
	WDG->PR = 31;
	//Preload value
	WDG->VR = timeout;
	//Wlacz uklad watchdog
	WDG->CR |= WDG_CR_WE;
}/*----------------------------------------------------------------------*/
/*----------------------------------------------------------------------*/
//Reset watchdog timer
static inline void reset_wdt(void)
{
	WDG->KR = WDG_MAGIC_KEY1;
	WDG->KR = WDG_MAGIC_KEY2;	
}

/*----------------------------------------------------------------------*/
//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    //Inicjalizuj timer WATCHDOG 2s
    setup_wdt(2048);	

    // petla nieskocznona
    while(1)
    {
	    int i;
	    for(i=0;i<8;i++)
    	    {
        	// Zalacz wylacz ledy
        	GPIO7->DR[0xff<<2]=(1<<i);
        	//Symulacja zawieszenia programu
        	if( !GPIO3->DR[0x80<<2] ) while(1);
		// czekaj chwile
		delay(1000000);
  		// resetuj watchdog
		reset_wdt();
    	    }
    }
    return 0;
}

