/*
    Str912 book example26 - UART transmit using DMA channel
    Copyright (C) 2008  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "91x_dma.h"
#include "91x_uart.h"
#include <stdio.h>
#include <string.h>
#include <stdint.h>
/*----------------------------------------------------------------------*/

//Main system clock is default 96MHz
#ifndef MAIN_CLOCK
#define MAIN_CLOCK 96000000LLU
#endif


/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}



/*----------------------------------------------------------------------*/
#define       DMA_Enable                        0x00000001
#define       DMA_ChannelEnable                 0x00000001
#define       DMA_ChannelDESInc                 0x08000000
#define       DMA_ChannelSRCInc                 0x04000000
#define       DMA_TCIE                          0x80000000
#define       UART_TXDMAE			0x0002


//Enable clocks for DMA controller
static void setup_uart_dma(void)
{
    //Reset kanalu DMA
    SCU->PRR0 &= ~__DMA;
    SCU->PRR0 |= __DMA;

    //Wlacz sygnal zegarowy dla DMA
    SCU->PCGR0 |= __DMA;

   //Enable DMA
    DMA->CNFR |= DMA_Enable;

    //Odblokuj zegar i reset dla UART0
    SCU->PRR1 |= __GPIO3 | __UART0;
    SCU->PCGR1 |= __GPIO3 | __UART0;

    //Ustaw porty tak aby RX bylo wejsciem a TX wyjsciem
    SCU->GPIOOUT[3] &= ~0x0C;
    SCU->GPIOOUT[3] |= (2<<(1*2));
    SCU->GPIOIN[3] |= (1<<0);
    GPIO3->DDR |= (1<<1);

    //Wyznaczanie mnoznika i czesciowego podzielnika
    const int baud = 64 * (MAIN_CLOCK /2) / (16 * 115200);
    UART0->IBRD = baud >> 6;
    UART0->FBRD = baud & 0x3f;

    //Konfiguruj urzadzenie z wlaczonym fifo
    UART0->LCR = UART_WordLength_8D |UART_FIFO_Enable;

    //Enable DMAREQ for Uart Transmit Data
    UART0->DMACR = UART_TXDMAE;

    //Wlacz nadawanie i odbieranie znakow
    UART0->CR = UART_Mode_Tx_Rx | 1;

    //Clear DMA interrupt
    DMA->TCICR = 1;
    DMA->EICR = 1;
}


/*----------------------------------------------------------------------*/
//Write DMA using
static int uart_dma_write(uint8_t *buf,int len)
{


    //Clear DMA interrupt
    DMA->TCICR = 1;
    DMA->EICR = 1;

    //Configuration register (Memory 2 device) and destination uart0 tx
    DMA_Channel0->CCNF = DMA_FlowCntrl1_DMA | DMA_DES_UART0_TX | DMA_ITMask_ITC;

    //Source memory area
    DMA_Channel0->SRC = (unsigned long)buf;

    //Destination memory area
    DMA_Channel0->DES = (unsigned long)&UART0->DR;

    //Linked list - not used
    DMA_Channel0->LLI = 0;

    //Configuration register
    DMA_Channel0->CC = DMA_DesWidth_Byte |    //Szerokosc danych wejciowych 1 bajt
                 DMA_SrcWidth_Byte |    //Szerokosc danych wyjsciowych 1 bajt
                 DMA_DesBst_1Data |     //Kopiowanie po 1 bajcie
                 DMA_SrcBst_4Data |     //Kopiowane po 4 bajty
                 DMA_ChannelSRCInc |    //Src increment
                 DMA_TCIE |             //End of transfer interrupt
                 (len & 0xfff);

    //Uaktywnij kanal DMA
    DMA_Channel0->CCNF |= DMA_ChannelEnable;

    //Wyjscie z funkcji
    return 0;
}


/*----------------------------------------------------------------------*/
/* Sprawdza status kanalu DMA:
    0 - NIC
    -1 - Blad
    1 - Koniec
*/

static int dma_status(int channel)
{
    //Jezeli blad to -1
    if( DMA->ERISR & (1<<channel) ) return -1;
    //Jezeli koniec transmisji to 1
    if( DMA->TCRISR & (1<<channel) ) return 1;
    return 0;
}

/*----------------------------------------------------------------------*/
uint8_t napis[] = "Ala ma kota a kot ma ale tralala\r\n";

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie PLL
    setup_pll();

    //Konfiguracja DMA
    setup_uart_dma();

    while(1)
    {
        uart_dma_write(napis,34);
        while(dma_status(0) == 0)
	{
	    //Tutaj procesor moze wykonywac inne czynnosci ;))
	}
        delay(1000000);
    }

    return 0;
}


