/*
    Str912 book example4 - Software Interrupt (System services)
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"


/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7 i 3
    SCU->PRR1 |= __GPIO7;

    //Wlacz sygnal zegarowy dla portu 7 i 3
    SCU->PCGR1 |= __GPIO7;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port 7 jako wyjsciowy
    GPIO7->DDR = 0xff;
    
    //Wylacz LEDY
    GPIO7->DR[0xff<<2] = 0;
}


/*----------------------------------------------------------------------*/
//Deklaracja funkcji przerwania SWI
void swiIntHandler(void) __attribute__ ((interrupt("SWI")));
//Funkcja przerwania SWI
void swiIntHandler(void) 
{
	//Rejestr R14-1 zawiera adres instrukcji SWI
	register unsigned int* link_ptr asm ("r14");
	//Rejestr R0 zawiera parametr przekazany do SWI
	register unsigned int param asm ("r0");
	switch(*(link_ptr-1) & 0x00FFFFFF)
	{
	//Zalacz Ledy
	case 0x01:
		GPIO7->DR[param<<2]=0xff;
		break;
	//Wylacz Ledy
	case 0x02:
		GPIO7->DR[param<<2]=0;
		break;
	}
}

/*----------------------------------------------------------------------*/
//Funkcja wlaczajaca LEDY poprzez SWI
static inline void swiLedOn(unsigned int LedSt)
{
	asm volatile 
	(
		"mov r0,%[ledon]\n"
		"swi #1\n"
		::[ledon]"r"(LedSt):"r0"
	);
}

/*----------------------------------------------------------------------*/
//Funkcja wylaczajaca LEDY poprzez SWI
static inline void swiLedOff(unsigned int LedSt)
{
	asm volatile 
	(
		"mov r0,%[ledon]\n"
		"swi #2\n"
		::[ledon]"r"(LedSt):"r0"
	);
}

/*----------------------------------------------------------------------*/
//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (zabezpieczenie JTAG)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

    // petla nieskocznona
    while(1)
    {
	//Wlacz ledy
	swiLedOn(0x93);
	//Opoznienie
	delay(5000000);
	//wylacz ledy
	swiLedOff(0x93);
	//Opoznienie
	delay(5000000);
    }
    return 0;
}

