/*
    Str912 book example8 - LCD Char C library
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "lcd.h"
#include <91x_map.h>
#include <91x_scu.h>
#include <stdint.h>
#include <stdbool.h>

/*----------------------------------------------------------*/
//Definicje poszczegolnych bitow
#define RS(n) GPIO9->DR[4<<2] = (n)<<2
#define RW(n) GPIO9->DR[2<<2] = (n)<<1
#define EN(n) GPIO9->DR[1<<2] = (n)<<0

#define DPORT GPIO8->DR[0xff<<2]

#define DPORT_IN false
#define DPORT_OUT true

//Przelacza port IO z wejscia na wyjscie i odwrotnie
static inline void dportDir(bool out)
{
    if(out) {SCU->GPIOOUT[8]  = 0x5555; GPIO8->DDR = 0xff; }
    else    {SCU->GPIOOUT[8]  = 0x0; GPIO8->DDR = 0x0; }
}


/*----------------------------------------------------------*/
//czeka n mikrosekund
static void delay_us(int us)
{
    us *= 50;
    while(us--)
	asm volatile ("nop");
}

/*----------------------------------------------------------*/
//Czeka sprawdzajac flage zajetosci
static void wait4lcd(void)
{
	uint8_t r;
	int tout = 16000;
	//Sprawdzenie czy jest wolny wyswietlacz
	do
	{
		EN(0);
		RS(0);
		RW(1);
		dportDir(DPORT_IN);
		EN(1);
		delay_us(1);
        r = DPORT;
		EN(0);
		if(--tout == 0) return;
	}
	while(r & 0x80);
	dportDir(DPORT_OUT);
}

/*----------------------------------------------------------*/

//Wysyla dana lub instrukcje w zaleznosci od id
static void write_lcd(uint8_t val,bool command)
{
    //Wlasciwy zapis danej do portu (Interfejs 4 bitowy)
	EN(0);
	RW(0);
	if(command) RS(0);
    else RS(1);
    DPORT = val;
	EN(1);
	delay_us(1);
    EN(0);
}


/*----------------------------------------------------------*/
//Inicjalizacja modulu LCD
void lcdInit(void)
{
    /** Inicjalizacja  GPIO **/
    // Wylacz sygnal resetu dla portow 8 i 9
    SCU->PRR1 |= __GPIO8 | __GPIO9;
    // Wlacz sygnal zegarowy dl portow 8 i 9
    SCU->PCGR1 |= __GPIO8 | __GPIO9;
    // Linie PUSH-PULL dla P8 i P9.0 .. P9.2
    SCU->GPIOTYPE[8] = 0x00;
    SCU->GPIOTYPE[9] &= ~0x07;
    // Linie portu 8 tymczasowo w kierunku wyjscia
    SCU->GPIOOUT[8]  = 0x5555;
    GPIO8->DDR = 0xff;
    // bity 0..2 portu w kierunku wyjscia
    uint16_t tmp = SCU->GPIOOUT[9];
    tmp  &= ~0x15;
    tmp  |= 0x15;
    SCU->GPIOOUT[9] = tmp;
    GPIO9->DDR |= 0x7;
    GPIO9->DDR |= 0xff;
    /** Inicjalizaca kontrolera LCD **/
    //Inicjalizacja Function Set
     write_lcd(0x30, 1); delay_us(200);
    //Inicjalizacja Function Set
     write_lcd(0x30, 1); delay_us(200);
    //Inicjalizacja Function Set
     write_lcd(0x30, 1); delay_us(200);
    //Function Set 2 linie znaki 5x7
     write_lcd(0x38, 1);
     wait4lcd();
	//Wyczysc wyswietlacz
     write_lcd(0x01, 1);
     wait4lcd();
	//Ustawienie kierunku zwiekszania linii
     write_lcd(0x06, 1);
     wait4lcd();
	//Przesuwanie kursora w prawo
     write_lcd(0x0c, 1);
     wait4lcd();
	//Ustawienie kursora na pozycji poczatkowej
     write_lcd(0x02, 1);
     wait4lcd();
    //Ustawienie adresu znakow na poczatek
     write_lcd(0x80, 1);
     wait4lcd();
}

/*----------------------------------------------------------*/
//Put string
void lcdPutStr(char *nap)
{
	while(*nap)
	{
		//Wyswietl znak
        write_lcd(*nap,0);
        //Czekaj
        wait4lcd();
	    //Przejdz do nastepnego znaku
        nap++;
	}
}

/*----------------------------------------------------------*/
//Wyczysc lcd
void lcdClear(void)
{
    //Wyslij komende czyszczenia
    write_lcd(1,1);
    //Czekaj
    wait4lcd();
}

/*----------------------------------------------------------*/
//Wyswietl znak
void lcdPutChar(char ch)
{
    //Wyslij literke
    write_lcd(ch,0);
    //Czekaj
    wait4lcd();
}

/*----------------------------------------------------------*/
//Wyswietl liczbe
void lcdPutInt(unsigned int num)
{
    char buf[10];
    unsigned int calk;
    int i=0;
    calk = num;
    while(calk)
    {
      calk /= 10;
      i++;
    }
    if(i)
    {
        calk = num;
        buf[i] = 0;
        for(--i;i>=0;i--)
        {
           buf[i] = calk % 10 + '0';
           calk /= 10;
        }
    }
    else
    {
        buf[0] = '0';
        buf[1] = 0;
    }
	char *str = buf;
	while(*str)
	{
        write_lcd(*str,0);
        wait4lcd();
        str++;
    }
}

/*----------------------------------------------------------*/
//Wyslij rozkaz
void lcdCommand(uint8_t disp,uint8_t blink,uint8_t cursor)
{
    write_lcd(disp+blink+cursor+0x08,1);
    wait4lcd();
}

/*----------------------------------------------------------*/
//Ustaw pozycje
void lcdSetPos(uint8_t pos)
{
    write_lcd(0x80 | pos,1);
    wait4lcd();
}


