/*
    Str912 book example9 - Simple uart library without interrupts
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "91x_map.h"
#include "91x_scu.h"
#include "91x_fmi.h"
#include "uart.h"
#include <string.h>
#include <stdlib.h>
/*----------------------------------------------------------------------*/
#define USED_UART UART0

/*----------------------------------------------------------------------*/

// delay for n cycles
static void delay(int n)
{
    while(n--)
	asm volatile ("nop");
}

/*----------------------------------------------------------------------*/

/* Ustawienie parametrow zegarowych */
//Czestotliwosc 96MHz
#define PLL_N 192
#define PLL_M 25
#define PLL_P 2
#define SCU_PLLEN 0x80000

static void  setup_pll(void)
{
    //Jezeli petla jest zsynchronizowana tzn ze jest ok
    if(SCU->SYSSTATUS & SCU_FLAG_LOCK) return;
    //Konfiguracja uruchomienie i synchronizacja PLL
    SCU->PLLCONF = (PLL_P<<16) | (PLL_N<<8) | PLL_M;
    SCU->PLLCONF |= SCU_PLLEN;
    while(~SCU->SYSSTATUS & SCU_FLAG_LOCK);

    //Ustawiamy pamiec flash na dwa cykle oczekiwania (dla 96MHz)
    FMI->CR |= FMI_WRITE_WAIT_STATE_1;
    *(vu16*)FMI_BANK_1 = 0x60;
    *(vu16*)(FMI_BANK_1|FMI_READ_WAIT_STATE_2|FMI_LVD_ENABLE|FMI_PWD_ENABLE|FMI_FREQ_HIGH) = 0x03;

    //Ustawiamy F clk na koncu bedzie z PLL-a
    SCU->CLKCNTR = SCU_EMIBCLK_Div2 | SCU_USBCLK_MCLK2 | SCU_PCLK_Div2;
}


/*----------------------------------------------------------------------*/
/* Ustawienie portow IO */
static void setup_gpio(void)
{
    // Wylacz sygnal zerowania dla portu 7 i 3
    SCU->PRR1 |= __GPIO7 | __GPIO3 ;

    //Wlacz sygnal zegarowy dla portu 7 i 3
    SCU->PCGR1 |= __GPIO7 | __GPIO3;

    //Wszystkie wyjscia portu 7 jako push pull
    SCU->GPIOTYPE[7] = 0x0000;

    //Wszystkie wyjscia portu 7 w trybie GPIO
    SCU->GPIOOUT[7]  = 0x5555;

    //Caly port 7 jako wyjsciowy
    GPIO7->DDR = 0xff;
    
    //Caly port P3.4..P3.7 jako wejscie
    SCU->GPIOIN[3] &= ~0xf0;
    
    //Port 3 jako wejsciowy
    GPIO3->DDR = 0;
}

/*----------------------------------------------------------------------*/

//Rozmiar bufora inbuf
#define INBUF_SIZE 255

//Bufor zawierajacy znaki wejsciowe
char inBuf[INBUF_SIZE+1];

//Licznik znakow bufora
uint8_t inBufCnt;
/*----------------------------------------------------------------------*/

//Funkcja glowna main
int main(void)
{
    //Poczatkowe opoznienie (JTAG CACHE)
    delay(1000000);

    //Ustawienie portow gpio
    setup_pll();

    //Ustawienie petli PLL (czestotliwosc 96MHz)
    setup_gpio();

	//Inicjalizacja portu szeregowego
	uartInit(USED_UART,115200);
	
	//Napis powitalny
	uartWriteString(USED_UART,"*** Mikrokontrolery STR912  - UART bez systemu przerwan***\n");
	uartWriteString(USED_UART,"*** (C) Lucjan Bryndza 2007. Licencja: GPLv3 ***\n");
	uartWriteString(USED_UART,"Program steruje diodami LED i sprawdza stan klawiszy\n");
	uartWriteString(USED_UART,"Aby zaswiecic lub wylaczyc diody wpisz setled [0-255]\n");
	uartWriteString(USED_UART,"Aby sprawdzic stan wcisnietych klawiszy wpisz key\n");
	uartWriteString(USED_UART,"\n>");
	
	
	while(1)
	{
		int zn = uartReadByte(USED_UART);
		if(zn!=UART_EMPTY)
		{
			uartWriteByte(USED_UART,zn);
			if(zn=='\r')
			{
				inBuf[inBufCnt] = 0;
				inBufCnt = 0;
				char *arg;
				if ((arg=strstr(inBuf,"setled"))!=NULL)
				{
					int leds = atoi(arg+7);
					if(leds)
					{
						GPIO7->DR[0xff<<2] = leds;
						uartWriteString(USED_UART,"Diody zostaly ustawione pomyslnie !!\n> ");
					}
					else
					{
						uartWriteString(USED_UART,"Bledny argument komendy !!\n> ");
					}
				}
				else if((arg=strstr(inBuf,"key"))!=NULL)
				{
					uartWriteString(USED_UART,"Klawisz K1 - ");
					if(!GPIO3->DR[0x10<<2])
					{
						uartWriteString(USED_UART,"WCISNIETY\n> ");
					}
					else
					{
						uartWriteString(USED_UART,"PUSZCZONY\n> ");
					}
					uartWriteString(USED_UART,"Klawisz K2 - ");
					if(!GPIO3->DR[0x20<<2])
					{
						uartWriteString(USED_UART,"WCISNIETY\n> ");
					}
					else
					{
						uartWriteString(USED_UART,"PUSZCZONY\n> ");
					}
					uartWriteString(USED_UART,"Klawisz K3 - ");
					if(!GPIO3->DR[0x40<<2])
					{
						uartWriteString(USED_UART,"WCISNIETY\n> ");
					}
					else
					{
						uartWriteString(USED_UART,"PUSZCZONY\n> ");
					}
					uartWriteString(USED_UART,"Klawisz K4 - ");
					if(!GPIO3->DR[0x80<<2])
					{
						uartWriteString(USED_UART,"WCISNIETY\n> ");
					}
					else
					{
						uartWriteString(USED_UART,"PUSZCZONY\n> ");
					}
				}
				else
				{
					uartWriteString(USED_UART,"Nieznana komenda !!\n> ");
				}
			}
			else
			{
				if(inBufCnt<INBUF_SIZE) inBuf[inBufCnt++] = zn;
			}
		
		}
	}
    return 0;
}

