/*
    Str912 book example9 - Simple uart library without interrupts
    Copyright (C) 2007  Lucjan Bryndza <lucjan.bryndza@ep.com.pl>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "uart.h"


/*-------------------------------------------------------------*/
//Initialize UART
int uartInit(UART_TypeDef *uartBase,int baud)
{
		
	if(uartBase==UART0)
	{
		//Odblokuj zegar i reset dla UART0
		SCU->PRR1 |= __GPIO3 | __UART0;
    		SCU->PCGR1 |= __GPIO3 | __UART0;
		uartBase = UART0;
		//Ustaw porty tak aby RX bylo wejsciem a TX wyjsciem
		SCU->GPIOOUT[3] &= ~0x0C;
		SCU->GPIOOUT[3] |= (2<<(1*2));
		SCU->GPIOIN[3] |= (1<<0);
		GPIO3->DDR |= (1<<1);
	}
	else if(uartBase==UART1)
	{
		//Odblokuj zegar i reset dla UART1
		SCU->PRR1 |= __GPIO3 | __UART1;
		SCU->PCGR1 |= __GPIO3 | __UART1;
		//Adres bazowt portu szeregowego
		uartBase = UART1;
		//Ustaw porty tak aby RX bylo wejsciem a TX wyjsciem
		SCU->GPIOOUT[3] &= 0xC0;
		SCU->GPIOOUT[3] |= (2<<(3*2));
		SCU->GPIOIN[3] |= (1<<2);
		GPIO3->DDR |= (3<<1);
	}
	else
	{
		return -1;
	}
	
	//Wyznaczanie mnoznika i czesciowego podzielnika
	baud = 64 * (MAIN_CLOCK /2) / (16 * baud);
	uartBase->IBRD = baud >> 6; 
	uartBase->FBRD = baud & 0x3f;
	//Konfiguruj urzadzenie z wlaczonym fifo
	uartBase->LCR = UART_WordLength_8D |UART_FIFO_Enable;
	//Wlacz nadawanie i odbieranie znakow
	uartBase->CR = UART_Mode_Tx_Rx | 1;
	return 0;
}

/*-------------------------------------------------------------*/
//Write character to UART
void uartWriteByte(UART_TypeDef *uartBase,uint8_t byte)
{
	// czekaj na miejsce w buforze
	while(uartBase->FR & (1<<5));
	// wyslij znak
	uartBase->DR = byte;
}

/*-------------------------------------------------------------*/
//Uart read byte
int uartReadByte(UART_TypeDef *uartBase)
{
// wait for incoming byte (FIFO status bit)
	if((uartBase->FR & (1<<4))) return UART_EMPTY; 

// read it from UART data register 
	return uartBase->DR & 0xff;
	
}

/*-------------------------------------------------------------*/
//Wyslij tekst do portu szeregowego 
void uartWriteString(UART_TypeDef *uartBase,const char *string)
{
	while(*string)
	{
		if(*string == '\n') uartWriteByte(uartBase,'\r');
		uartWriteByte(uartBase,*string++);
	}
}
